class OutputWriterPerConsensus(object):
    
    def outputWriter(self, outF, dResult):
        lMandatoryKeys = ['consensusName','consensusLength','consensusStrand','chimeric','consensusClass','consensusOrder','completeness']
        lKeys2Write = []
        lCodingAndStructAndOther2Write = []
        
        for key in lMandatoryKeys:
            lKeys2Write.append(dResult[key])

        confidenceIndex = dResult['confidenceIndex']
        confidenceIndex = 'CI=' + str(confidenceIndex)
        lKeys2Write.append(confidenceIndex)
        
        if dResult.has_key('dictOfEvidence'):
            dEvidence = dResult['dictOfEvidence']
            coding = self.writeCodingFeaturesLine(dEvidence)
            if coding != "":
                lCodingAndStructAndOther2Write.append(coding)
            
            struct = self.writeStructFeaturesLine(dEvidence)
            if struct != "":
                lCodingAndStructAndOther2Write.append(struct)
            
            other = self.writeOtherFeaturesLine(dEvidence)
            if other != "":
                lCodingAndStructAndOther2Write.append(other)
                
        outF.write('\t'.join(lKeys2Write) + "; " + "; ".join(lCodingAndStructAndOther2Write) + '\n')
        
    def writeOtherFeaturesLine(self, dEvidence): 
        other = ""
        if dEvidence.has_key('other'):
                lResults = []
                dOtherResults = dEvidence['other']
                lResultsWithCoding = self.formatCodingFeatures(dOtherResults, lResults)
                lResultsFilled = self.formatStructFeatures(dOtherResults, lResultsWithCoding)
                if len(lResultsFilled) != 0:
                    subOther = "; ".join(lResultsFilled)
                    other = 'other=(%s)' % subOther
        return other

    def writeCodingFeaturesLine(self, dEvidence):
        lResults = []
        lResultsFilled = self.formatCodingFeatures(dEvidence, lResults)
        coding = ""
        if len(lResultsFilled) != 0:
            subCoding = "; ".join(lResultsFilled)
            coding = 'coding=(%s)' % subCoding
        return coding

    def formatCodingFeatures(self, dEvidence, lResults):
        if dEvidence.has_key('Repbase_tbx') and dEvidence['Repbase_tbx'] != []:
            lResults.append("TE_BLRtx: %s" % ", ".join(map(str, dEvidence['Repbase_tbx'])))
        
        if dEvidence.has_key('Repbase_bx') and dEvidence['Repbase_bx'] != []:
            lResults.append("TE_BLRx: %s" % ", ".join(map(str, dEvidence['Repbase_bx'])))
            
        if (dEvidence.has_key('te_hmmer')) and (dEvidence['te_hmmer'] != None):
            lResults.append('profiles: %s' % self.formatProfilesResults(dEvidence['te_hmmer']))
            
        if dEvidence.has_key('Other_profiles'):
            lResults.append('Other_profiles: %s' % self.formatProfilesResults(dEvidence['Other_profiles']))
        
        if dEvidence.has_key("rDNA") and (dEvidence["rDNA"] != None):
            lResults.append("rDNA_BLRn: %s" % dEvidence["rDNA"])
        
        if dEvidence.has_key("HG") and (dEvidence["HG"] != None):
            lResults.append("HG_BLRn: %s" % dEvidence["HG"])
        
        return lResults

    def formatProfilesResults(self, dProfilesResults):
        if len(dProfilesResults.keys()) == 0:
            return ""
        lResults = []
        for key in dProfilesResults.keys():
            cov = "%.2f" % dProfilesResults[key]
            profilesResult = '%s: %s%s' % (key, cov, "%")
            lResults.append(profilesResult)
        return ", ".join(lResults)
    
    def writeStructFeaturesLine(self, dEvidence):
        lResults = []
        lResultsFilled = self.formatStructFeatures(dEvidence, lResults)
        struct = ""
        if len(lResultsFilled) != 0:
            subStruct = "; ".join(lResultsFilled)
            struct = 'struct=(%s)' % subStruct
        return struct

    def formatStructFeatures(self, dEvidence, lResults):
        if dEvidence.has_key('length') and (dEvidence['length']!= None):
            lResults.append('TElength: %s' % dEvidence['length'])

        if dEvidence.has_key('TR') and (dEvidence['TR'] != None):
            lResults.append('TermRepeats: %s' % ", ".join(map(str, dEvidence['TR'])))    
            
        if dEvidence.has_key('ORF') and (dEvidence['ORF'] != None):
            lResults.append('ORF: %s' % ", ".join(dEvidence['ORF']))        
            
        if dEvidence.has_key('SSRtrf') and (dEvidence['SSRtrf'] != None):
            lResults.append('SSRtrf: %s' % ", ".join(dEvidence['SSRtrf']))
        
        if dEvidence.has_key('SSRCoverage') and (dEvidence['SSRCoverage'] != None) :
            lResults.append('SSRCoverage=%s' % dEvidence['SSRCoverage'])
            
        if dEvidence.has_key('polyAtail'):
            lResults.append('polyAtail')   
             
        if dEvidence.has_key('helitronExtremities') and (dEvidence['helitronExtremities'] != None):
            lResults.append('helitronExtremities: %s' % ", ".join(map(str, dEvidence['helitronExtremities'])))
         
        return lResults