from PASTEC.utils.AbstractAnalyseAgent import AbstractAnalyseAgent
from PASTEC.Demand import Demand
from PASTEC.Size import Size
from PASTEC.DemandAndResult import DemandAndResult
import re

## This agent compute and categorize size of TE consensus without terminal repeats
# 
class Size_Agent(AbstractAnalyseAgent):
    
    TRSEARCH_AGENT = "TRSearch_Agent"
    SUFFIX_TABLE_NAME_LIST = []
    
    ## Constructor
    #
    # @param env Instance of Environment class
    #
    def __init__(self, env):
        self._dAgentName2AnalyserInstance = {}
        self._environment = env
        self._listOfKnownAgents = [self.TRSEARCH_AGENT]
            
    def analyse(self, demand):
        consensusName = demand.consensusName
        iTR = self.askTRsearchInfo(consensusName)
        consensusLengthFull = self._environment.getConsensusLength(consensusName)
        if iTR != None:
            consensusLengthWithoutTR = consensusLengthFull - iTR.getLength() * 2
        else:
            consensusLengthWithoutTR = consensusLengthFull
        iSize = Size(consensusLengthFull, consensusLengthWithoutTR)
        return DemandAndResult(demand, iSize)   
    
    ## Analyse accepted demands and add Result on black board results
    #        
    def live(self):
        for demand in self._environment.getBlackBoardDemands():
            if self.acceptDemand(demand):
                result = self.analyse(demand)
                self._environment.addResultToBlackBoard(result)
    
    ## To have information about term TR in the consensus
    # @param consensusName string name of TE consensus
    #
    def askTRsearchInfo(self, consensusName):
        if self._dAgentName2AnalyserInstance.has_key(self.TRSEARCH_AGENT):
            iDemand = Demand(self.TRSEARCH_AGENT, consensusName)
            if not self._environment.isDemandOnBlackBoardDemands(iDemand):
                self._environment.addDemandToBlackBoard(iDemand)
            iTRSearchAgent = self._dAgentName2AnalyserInstance[self.TRSEARCH_AGENT]
            iTRSearchAgent.createNeededAgentInstance()
            iTRSearchAgent.live()
            if self._environment.isDemandOnBlackBoardResults(iDemand):
                iDemandAndResult = self._environment.getResultAccordingToDemand(iDemand)
                lTR = iDemandAndResult.result
                for iTR in lTR:
                    if re.match('^term.*$', iTR.getType()):
                        return iTR
        return None