from PASTEC.utils.AbstractClassifierAdaptatorAgent import AbstractClassifierAdaptatorAgent
from PASTEC.Strand import Strand
from PASTEC.DemandAndResult import DemandAndResult
from PASTEC.Demand import Demand

MIN_BLASTER_COVERAGE = 30
MIN_PROFILES_COVERAGE = 75

class Strand_Agent(AbstractClassifierAdaptatorAgent):

    def __init__(self, env):
        self._environment = env
        self._dAgentName2AnalyserInstance = {}
        self._listOfKnownAgents = ["TE_BLRx_Agent", "TE_BLRtx_Agent", "TE_Profiles_Agent"]

    def getConclusionsMatch(self):
        return self._conclusionsMatch

    ## Set conclusionsMatch
    #  
    def setConclusionsMatch(self, conclusionsMatch):
        self._conclusionsMatch = conclusionsMatch
            
    def analyse(self, iDemand):
        strand = "."
        firstProfile = True
        maxBlasterCoverage = 0
        for result in self._conclusionsMatch:
            if self._isBlasterSignificantResult(result, maxBlasterCoverage):
                maxBlasterCoverage = result.getCoverage()
                strand = result.getStrand()
            elif self._isProfileSignificantResult(result, maxBlasterCoverage):
                if firstProfile:
                    strand = result.getStrand()
                    firstProfile = False
                elif strand != result.getStrand():
                    strand = "."
        return DemandAndResult(iDemand, Strand(strand))
    
    def _isBlasterSignificantResult(self, result, maxBlasterCoverage):
        return str(result.__class__.__name__) == "BlasterMatch" and result.getCoverage() > maxBlasterCoverage and result.getCoverage() > MIN_BLASTER_COVERAGE
    
    def _isProfileSignificantResult(self, result, maxBlasterCoverage):
        return maxBlasterCoverage == 0 and result.getCoverage() > MIN_PROFILES_COVERAGE
        
    ## Three main step : make demands, get results and decide
    #
    def live(self):
        for demand in self._environment.getBlackBoardDemands():
            if self.acceptDemand(demand):
                self._consensusName = demand.consensusName 
                self._initializeConclusions()
                self._makeDemands()               

                self.getAnalysesResults()
                iDemandAndResult = self.analyse(demand)
                
                self._environment.addResultToBlackBoard(iDemandAndResult)
    
    def _initializeConclusions(self):
        self._conclusionsMatch = []

    def _callAgent (self, AgentInstance):
        AgentInstance.createNeededAgentInstance()
        AgentInstance.live()
        iDemand = Demand(AgentInstance.__class__.__name__, self._consensusName)
        try:
            iDemandAndResult = self._environment.getResultAccordingToDemand(iDemand)
        except:
            return None
        return iDemandAndResult
    
    def makeAppropriateConclusionsForTE_BLRx_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != []:
            self._conclusionsMatch.extend(iDemandAndResult.result)
    
    def makeAppropriateConclusionsForTE_BLRtx_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != []:
            self._conclusionsMatch.extend(iDemandAndResult.result)
    
    def makeAppropriateConclusionsForTE_Profiles_Agent(self, iDemandAndResult):
        if iDemandAndResult != None and iDemandAndResult.result != []:
            self._conclusionsMatch.extend(iDemandAndResult.result)