from PASTEC.utils.IAgent import IAgent
from PASTEC.utils.AgentFactory import AgentFactory

## Use this class to create a new analyser agent.

class AbstractAnalyseAgent(IAgent):
    
    ## Constructor
    #
    # @param env Instance of Environment class
    # @param projectName string name of project
    #
    def __init__(self, env):
        self._db = env.getDbInstance()
        self._environment = env
        self.setTableName()
        self._listOfKnownAgents = []

    ## Set table name
    #
    def setTableName(self): 
        self._tableName = self._environment.getProjectName() + self.getSuffixTableNameList()[0]
        
    ## Get suffix table name list
    #
    # @return list of string SUFFIX_TABLE_NAME_LIST
    #
    def getSuffixTableNameList(self):
        return self.SUFFIX_TABLE_NAME_LIST
        
    ## Accept or not a demand_conclusionsMatch
    #
    # @param demand a Demand object type (name of Agent and name of sequence to analyse for example)
    # @return True if demand is accepted, false otherwise
    #
    def acceptDemand(self, demand):
        return demand.agent == self.__class__.__name__\
            and not self._environment.isDemandOnBlackBoardResults(demand)\
            and self._environment.isConsensusInEnvironment(demand.consensusName)
    
    ## Analyse accepted demands and add Result on black board results
    #        
    def live(self):
        for demand in self._environment.getBlackBoardDemands():
            if self.acceptDemand(demand):
                lOfFeatures = self.getListOfFeaturesToAnalyse(demand.consensusName)
                result = self.analyse(demand, lOfFeatures)
                self._environment.addResultToBlackBoard(result)
                
    ## get size of the consensus
    # @param consensusName string name of TE consensus
    # @return integer size of TE consensus
    #
    def askForConsensusLength(self, consensusName):
        return self._environment.getConsensusLength(consensusName)
    
    ## Create Needed Agent instances if corresponding table is in database.
    # _dAgentName2AnalyserInstance is filled with analyser agent class name as key
    # and associated instance as value
    #
    def createNeededAgentInstance(self):
        for knownAgentName in self._listOfKnownAgents:
            self.createOneNeededAgentInstance(knownAgentName)
    
    ## Create Needed Agent instance if corresponding table is in database.
    # _dAgentName2AnalyserInstance is filled with analyser agent class name as key
    # and associated instance as value
    #
    # @param agentName string name of agent desired
    #
    def createOneNeededAgentInstance(self, agentName):
        currentAgentNewInstance = AgentFactory.createAgent(agentName, self._environment)
        lTableSuffix = currentAgentNewInstance.getSuffixTableNameList()
        isCreateInstance = True
        for tableSuffix in lTableSuffix:
            tableName = self._environment.getProjectName() + tableSuffix
            if tableName not in self._environment.getTablesList():
                isCreateInstance = False
                break
        if isCreateInstance:
            self._dAgentName2AnalyserInstance [currentAgentNewInstance.__class__.__name__] = currentAgentNewInstance
            
    ## get useful informations in database
    #
    # @param consensusName string name of consensus
    #
    def getListOfFeaturesToAnalyse(self, consensusName):
        pass
    
    ## Analyse a Demand (return a Result)
    #
    # @param demand a Demand object type (name of Agent and name of sequence to analyse for example)
    #
    def analyse(self, demand):
        pass
        
    def patch(self, result):
        return result