#!/usr/bin/env python

# Copyright INRA (Institut National de la Recherche Agronomique)
# http://www.inra.fr
# http://urgi.versailles.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


import os
import sys
import time
import shutil
import ConfigParser
import glob
import optparse
import subprocess

if not "REPET_PATH" in os.environ.keys():
    print "ERROR: no environment variable REPET_PATH"
    sys.exit(1)
sys.path.append(os.environ["REPET_PATH"])
if not "PYTHONPATH" in os.environ.keys():
    os.environ["PYTHONPATH"] = os.environ["REPET_PATH"]
else:
    os.environ["PYTHONPATH"] = "%s:%s" % (os.environ["REPET_PATH"], os.environ["PYTHONPATH"])

from ConfigParser import NoOptionError
from ConfigParser import NoSectionError
from ConfigParser import MissingSectionHeaderError
from pyRepet.launcher.programLauncher import programLauncher
from pyRepetUnit.commons.launcher.Launcher import Launcher
from pyRepetUnit.commons.utils.FileUtils import FileUtils
from pyRepetUnit.commons.checker.CheckerUtils import CheckerUtils
from pyRepetUnit.commons.checker.CheckerException import CheckerException
from pyRepetUnit.commons.checker.ConfigChecker import ConfigChecker
from pyRepetUnit.commons.checker.ConfigChecker import ConfigRules
from pyRepetUnit.commons.seq.FastaUtils import FastaUtils
from pyRepetUnit.commons.coord.Align import Align
from pyRepetUnit.commons.sql.RepetJob import RepetJob
from pyRepetUnit.commons.sql.TableJobAdaptatorFactory import TableJobAdaptatorFactory
from pyRepetUnit.commons.sql.DbFactory import DbFactory
from denovo_pipe.RemoveRedundancyBasedOnClassif import RemoveRedundancyBasedOnClassif
from denovo_pipe.RenameHeaderClassif import RenameHeaderClassif
from repet_tools.ReverseComplementAccordingToClassif import ReverseComplementAccordingToClassif
from repet_tools.GetClassifUniq import GetClassifUniq


#TODO : update help and verbosity
#------------------------------------------------------------------------------

## Setup the required environment.
#
# @param config ConfigParser instance 
#
def setup_env(config):
    os.environ["REPET_HOST"] = config.get("repet_env", "repet_host")
    os.environ["REPET_USER"] = config.get("repet_env", "repet_user")
    os.environ["REPET_PW"] = config.get("repet_env", "repet_pw")
    os.environ["REPET_DB"] = config.get("repet_env", "repet_db")
    os.environ["REPET_PORT"] = config.get("repet_env", "repet_port")
    os.environ["REPET_JOB_MANAGER"] = config.get("repet_env", "repet_job_manager")
    os.environ["REPET_QUEUE"] = config.get("repet_env", "repet_job_manager")
    os.environ["REPET_JOBS"] = "MySQL"

#------------------------------------------------------------------------------

def help():
#TODO: useful for sys.exit(1)... => to change
    print
    print "Wrong options."
    print "'TEdenovo.py -h' for more informations"
    print

#------------------------------------------------------------------------------

def main():

    """
    This program is a pipeline for the de novo identification of transposable elements (TEs) in genomic sequences.
    """

    description = "Launch TEdenovo pipeline"
    examples = "EXAMPLE TODO"
    parser = optparse.OptionParser(description = description, version = "v1.0", epilog = examples)
    parser.add_option( "-P", "--project", dest = "projectName", action="store", type="string", default = "", help = "project name (<=15 symbols, alphanumeric or underscore)")
    parser.add_option( "-C", "--config", dest = "configFileName", action="store", type="string", default = "", help = "configuration file")
    parser.add_option( "-S", "--step", dest = "step", action="store", type="string", default = "", help = "step (1/2/3/4/5/6/7/8)")
    parser.add_option( "", "--struct", dest = "isStructuralSearch", action="store_true", default = False, help = "search for LTR using LTRharvest (default=False)")
    parser.add_option( "-s", "--align", dest = "smplAlign", action="store", type="string", default = "", help = "program to self-align (Blaster)")
    parser.add_option( "-c", "--clust", dest = "clustHsp", action="store", type="string", default = "", help = "program to cluster HSPs (Grouper/Recon/Piler or GrpRec/GrpPil/RecPil/GrpRecPil)")
    parser.add_option( "-m", "--msa", dest = "multAlign", action="store", type="string", default = "", help = "program to build MSAs (Map)")
    parser.add_option( "-v", "--verbose", dest = "verbose", action="store", type="int", default = 1, help = "verbose (0/default=1)")
    (options, args) = parser.parse_args()

    global projectName
    projectName = options.projectName
    global configFileName
    configFileName = options.configFileName
    step = options.step
    smplAlign = options.smplAlign
    clustHsp = options.clustHsp
    multAlign = options.multAlign
    global verbose
    verbose = options.verbose
    isStructuralSearch = options.isStructuralSearch
    
    global configFileHandle

    if projectName == "" or configFileName == "" or step == "":
        print "ERROR: missing compulsory options"
        help()
        sys.exit(1)

    if not CheckerUtils.isMax15Char(projectName):
        print "ERROR: project name must have 15 character max"
        help()
        sys.exit(1)
        
    if not CheckerUtils.isCharAlphanumOrUnderscore(projectName):
        print "ERROR: project name must contain only alphanumeric or underscore character"
        help()
        sys.exit(1)

    global config
    configFileHandle = open(configFileName)
    config = ConfigParser.ConfigParser()
    
    try :
        config.readfp( configFileHandle )
    except MissingSectionHeaderError:
        print "ERROR: config file " + configFileName + " must begin with a section header "  
        sys.exit(1)
    
    isOptionErrorRaised = False
    sectionName = "repet_env"
    try:
        CheckerUtils.checkSectionInConfigFile(config, sectionName)
    except NoSectionError:
        print "ERROR: the section " + sectionName + " must be in your config file : " + configFileName
        sys.exit(1)

    optionName = "repet_version"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " +  optionName +" must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True 
    optionName = "repet_host"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " +  optionName +" must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True 
            
    optionName = "repet_user"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " +  optionName +" must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True 
            
    optionName = "repet_pw"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " +  optionName +" must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True 
            
    optionName = "repet_db"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " +  optionName +" must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True 
            
    optionName = "repet_port"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " +  optionName +" must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True 
        
    if isOptionErrorRaised:
        sys.exit(1)

        
    setup_env( config )
    
    changeLogFileName = os.environ["REPET_PATH"] + "/CHANGELOG" 
    changeLogFileHandle = open(changeLogFileName, "r")
    
    try:
        CheckerUtils.checkConfigVersion(changeLogFileHandle, config)
    except CheckerException, e:
        print e.msg
        sys.exit(1)
    
    sectionName = "project"
    
    try:
        CheckerUtils.checkSectionInConfigFile(config, sectionName)
    except NoSectionError:
        print "ERROR: the section " + sectionName + " must be in your config file : " + configFileName
        sys.exit(1)
    
    isOptionErrorRaised = False
    optionName = "project_name"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " +  optionName +" must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True    

    optionName = "project_dir"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " +  optionName +" must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True    

    if isOptionErrorRaised :
        sys.exit(1)


    if verbose > 0:
        print "\nSTART TEdenovo.py (%s)" % ( time.strftime("%Y-%m-%d %H:%M:%S") )
        print "version %s" % ( config.get("repet_env","repet_version") )
        sys.stdout.flush()

    if config.get( "project", "project_name" ) != projectName:
        print "ERROR: project name different between configuration file and command-line option"
        sys.exit(1)
    if verbose > 0:
        print "project name = %s" % ( projectName ); sys.stdout.flush()

    global projectDir
    projectDir = config.get(sectionName, "project_dir" )
    if projectDir[-1] == "/":
        projectDir = projectDir[:-1]
    if os.path.exists( projectDir ):
        if verbose > 0:
            print "project directory = %s" % ( projectDir ); sys.stdout.flush()
        os.chdir( projectDir )
    else:
        print "ERROR: project directory %s doesn't exist" % ( projectDir )
        sys.exit(1)

    global pL
    pL = programLauncher()

    if step == "1":
        iPrepareBatches = PrepareBatches(projectDir, projectName, config, verbose)
        iPrepareBatches.run()

    if step == "2" and not isStructuralSearch:
        selfAlign( smplAlign )

    if step == "3" and not isStructuralSearch:
        clusterHsp( smplAlign, clustHsp )

    if step == "2" and isStructuralSearch:
        iTEStructuralSearch = TEStructuralSearch(projectDir, projectName, config, verbose)
        iTEStructuralSearch.run()
        
    if step == "3" and isStructuralSearch:
        iTEClustering = TEClustering(projectDir, projectName, config, verbose)
        iTEClustering.run()

    if step == "4":
        buildConsensus( smplAlign, clustHsp, multAlign, isStructuralSearch )

    if step == "5":
        if isStructuralSearch and smplAlign == "":
            smplAlign = "LTRharvest"
            clustHsp = "Blastclust"
        detectFeatures( smplAlign, clustHsp, multAlign, isStructuralSearch )

    if step == "6":
        if isStructuralSearch and smplAlign == "":
            smplAlign = "LTRharvest"
            clustHsp = "Blastclust"
        iClassifConsensus = ClassifConsensus(smplAlign, clustHsp, multAlign, isStructuralSearch, configFileName, verbose)
        iClassifConsensus.run()

    if step == "7":
        if isStructuralSearch and smplAlign == "":
            smplAlign = "LTRharvest"
            clustHsp = "Blastclust"
        filterConsensus( smplAlign, clustHsp, multAlign, isStructuralSearch )

    if step == "8":
        if isStructuralSearch and smplAlign == "":
            smplAlign = "LTRharvest"
            clustHsp = "Blastclust"
        clusteringConsensus( smplAlign, clustHsp, multAlign, isStructuralSearch )

    if verbose > 0:
        print "version %s" % config.get("repet_env","repet_version")
        print "END TEdenovo.py (%s)\n" % time.strftime("%Y-%m-%d %H:%M:%S")
        sys.stdout.flush()

    return 0
#------------------------------------------------------------------------------

class PrepareBatches(object):
    
    def __init__(self, projectDir, projectName, iConfig, verbose):
        self._projectDir = projectDir
        self._projectName = projectName
        self._iConfig = iConfig
        self._verbose = verbose
        
    def run(self):
        if verbose > 0:
            print "beginning of step 1"
            sys.stdout.flush()
        if FileUtils.isRessourceExists("%s_db" % self._projectName):
            print "ERROR: directory '%s_db' already exists" % self._projectName
            sys.exit(1)
        
        os.mkdir("%s_db" % self._projectName)
        os.chdir("%s_db" % self._projectName)
        genomeFastaFileName = "%s.fa" % self._projectName
        os.symlink("../%s" % genomeFastaFileName, genomeFastaFileName)
        sectionName = "prepare_batches"
        self._checkConfig(sectionName)  
        
        separator = "\n"
        inGenomeFileHandler = open(genomeFastaFileName, "r")
        try:
            CheckerUtils.checkHeaders(inGenomeFileHandler)
        except CheckerException, e:
            print "Error in file %s. Wrong headers are :" % genomeFastaFileName
            print separator.join(e.messages)
            print "Authorized characters are : a-z A-Z 0-9 - . : _\n"
            inGenomeFileHandler.close()
            sys.exit(1)
        inGenomeFileHandler.close()
        
        # build the chunks and save the chunks into batch files
        self._launchCommands(sectionName)
        
        os.chdir( ".." )
        if verbose > 0:
            print "step 1 finished successfully"
            sys.stdout.flush()

    def _checkConfig(self, sectionName):
        try:
            CheckerUtils.checkSectionInConfigFile(self._iConfig, sectionName)
        except NoSectionError:
            print "ERROR: the section %s must be in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "chunk_length")
        except NoOptionError:
            print "ERROR: the option 'chunk_length' must be define in %s in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "chunk_overlap")
        except NoOptionError:
            print "ERROR: the option 'chunk_overlap' must be define in %s in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "nb_seq_per_batch")
        except NoOptionError:
            print "ERROR: the option 'nb_seq_per_batch' must be define in %s in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "resources")
        except NoOptionError:
            print "ERROR: the option 'resources' must be define in %s in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "tmpDir")
        except NoOptionError:
            print "ERROR: the option 'tmpDir' must be define in %s in your configuration file" % sectionName
            sys.exit(1)

    def _createDbChunksCommand(self, sectionName, iLauncher):
        inFile =  "%s.fa" % self._projectName
        outputFile = "%s_chunks" % self._projectName
        lArgs = []
        lArgs.append("-i %s" % inFile)
        lArgs.append("-l %s" % self._iConfig.get(sectionName, "chunk_length"))
        lArgs.append("-o %s" % self._iConfig.get(sectionName, "chunk_overlap"))
        lArgs.append("-w 0")
        lArgs.append("-O %s" % outputFile)
        lArgs.append("-c")
        lArgs.append("-v %i" % (verbose - 1))
        if self._iConfig.get(sectionName, "clean") == "yes":
            lArgs.append("-c")
        
        return iLauncher.getSystemCommand("%s/bin/dbChunks.py" % os.environ["REPET_PATH"], lArgs)
        
    def _createDbSplitCommand(self, sectionName, iLauncher):
        inFile =  "%s_chunks.fa" % self._projectName
        lArgs = []
        lArgs.append("-i %s" % inFile)
        lArgs.append("-n %s" % self._iConfig.get(sectionName, "nb_seq_per_batch"))
        lArgs.append("-d")
        lArgs.append("-v %i" % (verbose - 1))
        
        return iLauncher.getSystemCommand("%s/bin/dbSplit.py" % os.environ["REPET_PATH"], lArgs)

    def _launchCommands(self, sectionName):
        resources = self._iConfig.get(sectionName, "resources")
        cDir = os.getcwd()
        if self._iConfig.get(sectionName, "tmpDir") != "":
            tmpDir = self._iConfig.get(sectionName, "tmpDir")
        else:
            tmpDir = cDir
        groupid = "%s_TEdenovo_prepareBatches" % self._projectName
        acronym = "dbChunks_dbSplit"
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", resources, groupid, acronym)
        lCmds = []
        lCmds.append(self._createDbChunksCommand(sectionName, iLauncher))
        lCmds.append(self._createDbSplitCommand(sectionName, iLauncher))
        lCmdStart = []
        lCmdStart.append("os.symlink('%s/%s.fa', '%s.fa')\n" % (cDir, self._projectName, self._projectName))
        lCmdFinish = []
        lCmdFinish.append("shutil.move('%s_chunks.fa', '%s/.')\n" % (self._projectName, cDir))
        lCmdFinish.append("shutil.move('%s_chunks.map', '%s/.')\n" % (self._projectName, cDir))
        lCmdFinish.append("shutil.move('batches', '%s/batches')\n" % cDir)
        lCmdsTuples = []
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        cleanMustBeDone = False
        if config.get(sectionName, "clean") == "yes":
            cleanMustBeDone = True
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)
#------------------------------------------------------------------------------

class TEStructuralSearch(object):
    
    def __init__(self, projectDir, projectName, iConfig, verbose):
        self._projectDir = projectDir
        self._projectName = projectName
        self._iConfig = iConfig
        self._verbose = verbose
    
    def run(self):
        if verbose > 0:
            print "beginning of step 2 (LTRharvest)"
            sys.stdout.flush()
        if FileUtils.isRessourceExists("%s_LTRharvest" % self._projectName):
            print "ERROR: directory '%s_LTRharvest' already exists" % self._projectName
            sys.exit(1)
        
        os.mkdir("%s_LTRharvest" % self._projectName)
        os.chdir("%s_LTRharvest" % self._projectName)
        sectionName = "structural_search"
        self._checkConfig(sectionName)  

        resources = self._iConfig.get(sectionName, "resources")
        cDir = os.getcwd()
        if self._iConfig.get(sectionName, "tmpDir") != "":
            tmpDir = self._iConfig.get(sectionName, "tmpDir")
        else:
            tmpDir = cDir
            
        similarityLTR = int(self._iConfig.get(sectionName, "LTR_similarity"))
        
        groupid = "%s_TEdenovo_LTRharvest" % self._projectName
        acronym = "LTRharvest"
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", resources, groupid, acronym)
        
        lCmdsTuples = []
        lFiles = FileUtils.getFileNamesList("../%s_db/batches" % self._projectName, "batch*")
        for file in lFiles:
            lCmds = []
            lCmds.append(self._createLTRharvestCommand(file, similarityLTR, iLauncher))
            lCmdStart = []
            lCmdStart.append("shutil.copy('%s/%s_db/batches/%s', '.' )" % (self._projectDir, self._projectName, file))
            lCmdFinish = []
            lCmdFinish.append("shutil.move('%s_LTRharvest.fa', '%s')" % (os.path.splitext(file)[0], cDir))
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
            
        cleanMustBeDone = False
        if config.get(sectionName, "clean") == "yes":
            cleanMustBeDone = True    
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)
        
        #Concatenate all LTRharvest prediction in a single file
        FileUtils.catFilesByPattern( "*.fa", "%s_LTRharvest.fa" % self._projectName)
        if cleanMustBeDone:
            FileUtils.removeFilesByPattern("batch*")
        
        os.chdir( ".." )
        if verbose > 0:
            print "step 2 (LTRharvest) finished successfully"
            sys.stdout.flush()

    def _createLTRharvestCommand(self, file, similarity, iLauncher):
        lArgs = []
        lArgs.append("--db %s" % file)
        lArgs.append("--sim %s" %  similarity)
        return iLauncher.getSystemCommand("LaunchLTRharvest.py", lArgs)

    def _checkConfig(self, sectionName):
        try:
            CheckerUtils.checkSectionInConfigFile(self._iConfig, sectionName)
        except NoSectionError:
            print "ERROR: the section %s must be in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "resources")
        except NoOptionError:
            print "ERROR: the option 'resources' must be define in %s in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "tmpDir")
        except NoOptionError:
            print "ERROR: the option 'tmpDir' must be define in %s in your configuration file" % sectionName
            sys.exit(1)

#------------------------------------------------------------------------------

class TEClustering(object):
    
    def __init__(self, projectDir, projectName, iConfig, verbose):
        self._projectDir = projectDir
        self._projectName = projectName
        self._iConfig = iConfig
        self._verbose = verbose
        self._clustHsp = "Blastclust"
        
    def run(self):
        if self._verbose > 0:
            print "beginning of step 3 (Blastclust)"
            sys.stdout.flush()
        if FileUtils.isRessourceExists("%s_LTRharvest_Blastclust" % self._projectName):
            print "ERROR: directory '%s_LTRharvest_Blastclust' already exists" % self._projectName
            sys.exit(1)
        
        os.mkdir("%s_LTRharvest_Blastclust" % self._projectName)
        os.chdir("%s_LTRharvest_Blastclust" % self._projectName)
        sectionName = "structural_search"
        self._checkConfig(sectionName)  
        
        mapFilename = "%s_%s_%s.map" % (projectName, "LTRharvest", self._clustHsp)
                                      
        blastclustOptions = "-b F -o %s_LTRharvest -m %s" % (self._projectName, mapFilename)
        
        resources = self._iConfig.get(sectionName, "resources")
        cDir = os.getcwd()
        if self._iConfig.get(sectionName, "tmpDir") != "":
            tmpDir = self._iConfig.get(sectionName, "tmpDir")
        else:
            tmpDir = cDir
        groupid = "%s_TEdenovo_LTRharvest_Blastclust" % self._projectName
        
        jobdb = RepetJob()
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", resources, groupid)

        fileName = "%s_LTRharvest.fa" % self._projectName
        blastclustFileName = "%s_Blastclust.fa" % os.path.splitext(fileName)[0]
        blastclustMapFileName = "%s_Blastclust.map" % os.path.splitext(fileName)[0]
        
        lCmds = [self._createBlastclustCommand(fileName, iLauncher, blastclustOptions)]
        
        lCmdStart = ["shutil.copy('%s/%s_LTRharvest/%s', '.' )" % (self._projectDir, self._projectName, fileName)]
        lCmdFinish = ["shutil.move('%s_blastclust.fa', '%s/%s')" % (os.path.splitext(fileName)[0],  cDir, blastclustFileName)]
        lCmdFinish.append("shutil.move('%s_blastclust.map', '%s/%s')" % (os.path.splitext(fileName)[0],  cDir, blastclustMapFileName))
        lCmdsTuples = [iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish)]
            
        cleanMustBeDone = False
        if config.get(sectionName, "clean") == "yes":
            cleanMustBeDone = True    
        iLauncher.runLauncherForMultipleJobs("Blastclust", lCmdsTuples, cleanMustBeDone)

        #Clusters filtering:
        minSeq = 1
        maxSeq = 20
        
        prg = os.environ["REPET_PATH"] + "/bin/FilterSeqClusters.py"
        cmdFilter = prg
        cmdFilter += " -i %s" % mapFilename
        cmdFilter += " -c %s" % self._clustHsp
        cmdFilter += " -m %s" % minSeq
        cmdFilter += " -M %s" % maxSeq
        cmdFilter += " -L -1"
        cmdFilter += " -v %i" % (self._verbose - 1)
        
        os.system(cmdFilter)
        
        input = "%s_%s_%s.map.filtered-%s-%s" % (projectName, "LTRharvest", self._clustHsp, minSeq, maxSeq)     
        
        bankName = projectName
        bankName += "_chunks.fa"
        pathToFasta = "%s/%s_db/%s" % (projectDir, projectName, bankName)
        
        prg = "%s/bin/map2db " % os.environ["REPET_PATH"]
        cmd = prg
        cmd += input
        cmd += " %s" % pathToFasta
        
        os.system(cmd)
        
        os.symlink("%s.filtered-%s-%s.flank_size0.fa" % (mapFilename, minSeq, maxSeq), "%s_LTRharvest_Blastclust_%selem_%sseq.fa" % (self._projectName, minSeq, maxSeq))
        os.chdir("..")
        if self._verbose > 0:
            print "step 3 (Blastclust) finished successfully"
            sys.stdout.flush()

    def _createBlastclustCommand(self, file, iLauncher, blastclustOptions):
        lArgs = []
        lArgs.append("-i %s" % file)
        lArgs.append(blastclustOptions)
        return iLauncher.getSystemCommand("LaunchBlastclust.py", lArgs)

    def _checkConfig(self, sectionName):
        try:
            CheckerUtils.checkSectionInConfigFile(self._iConfig, sectionName)
        except NoSectionError:
            print "ERROR: the section %s must be in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "resources")
        except NoOptionError:
            print "ERROR: the option 'resources' must be define in %s in your configuration file" % sectionName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(self._iConfig, sectionName, "tmpDir")
        except NoOptionError:
            print "ERROR: the option 'tmpDir' must be define in %s in your configuration file" % sectionName
            sys.exit(1)

#------------------------------------------------------------------------------

def getBlasterCmd(verbose, projectName, sectionName, config, iLauncher, file):
    lArgs = []
    lArgs.append("-q %s" % file)
    lArgs.append("-s %s_chunks.fa" % projectName)
    lArgs.append("-B %s" % file)
    lArgs.append("-a")
    lArgs.append("-E %s" % config.get(sectionName, "Evalue"))
    lArgs.append("-L %s" % config.get(sectionName, "length"))
    lArgs.append("-I %s" % config.get(sectionName, "identity"))
    if config.get(sectionName, "blast") == "ncbi": 
        lArgs.append("-N")
        lArgs.append("-p '-a 1'")
    elif config.get(sectionName, "blast") == "wu":
        lArgs.append("-W")
        lArgs.append("-p -cpus=1")
    elif config.get(sectionName, "blast") == "blastplus":
        lArgs.append("-X")
        lArgs.append("-p '-num_threads 1'")
# TODO: check the check option at the beginning of step 2 to allow to launch megablast for blast and blast+
#    elif config.get(sectionName, "blast") == "mega":
#        lArgs.append("-N")
#        lArgs.append("-n megablast")
#    elif config.get(sectionName, "blast") == "megablastplus":
#        lArgs.append("-X")
#        lArgs.append("-n megablast")
        
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand("blaster", lArgs)

def filterRedundantMatches(inFile, outFile):
    """
    When a pairwise alignment is launched ~ all-by-all (ie one batch against all chunks),
    one filters the redundant matches. For instance we keep 'chunk3-1-100-chunk7-11-110-...'
    and we discards 'chunk7-11-110-chunk3-1-100-...'.
    Also we keep 'chunk5-1-100-chunk5-11-110-...' and we discards
    'chunk5-11-110-chunk5-1-100-...'.
    For this of course the results need to be sorted by query, on plus strand,
    and in ascending coordinates (always the case with Blaster).
    """
    inFileHandler = open(inFile, "r")
    outFileHandler = open(outFile, "w")
    iAlign = Align()
    countMatches = 0
    tick = 100000
    while True:
        line = inFileHandler.readline()
        if line == "":
            break
        countMatches += 1
        iAlign.setFromString( line )
        if "chunk" not in iAlign.range_query.seqname \
               or "chunk" not in iAlign.range_subject.seqname:
            print "ERROR: 'chunk' not in seqname"
            sys.exit(1)
        if int(iAlign.range_query.seqname.split("chunk")[1]) < int(iAlign.range_subject.seqname.split("chunk")[1]):
            iAlign.write( outFileHandler )
        elif int(iAlign.range_query.seqname.split("chunk")[1]) == int(iAlign.range_subject.seqname.split("chunk")[1]):
            if iAlign.range_query.getMin() < iAlign.range_subject.getMin():
                iAlign.write( outFileHandler )
        if countMatches % tick == 0:   # need to free buffer frequently as file can be big
            outFileHandler.flush()
            os.fsync( outFileHandler.fileno() )
    inFileHandler.close()
    outFileHandler.close()

def runCollect(groupid, collect, allByAll, clean):
    """
    Gather the results of each job in a single job and adapt path ID if necessary.
    """
    if verbose > 0:
        print "concatenate the results of each job"
        sys.stdout.flush()

    # retrieve the list of the files
    lFiles = glob.glob("*.%s" % collect)
    lFiles.sort()

    # concatenate all the individual files
    if os.path.exists( "%s.%s_tmp" % (groupid, collect) ):
        os.remove( "%s.%s_tmp" % (groupid, collect) )
    for resFile in lFiles:
        prg = "cat"
        cmd = prg
        cmd += " %s >> %s.%s_tmp" % (resFile, groupid, collect)
        pL.launch(prg, cmd)
        if clean:
            prg = "rm"
            cmd = prg
            cmd += " -f %s" % resFile
            pL.launch(prg, cmd)

    if os.path.exists("%s.%s" % (groupid, collect) ):
        os.remove("%s.%s" % (groupid, collect) )

    if not allByAll:
        shutil.move("%s.%s_tmp", "%s.%s" % (groupid, collect, groupid, collect) )
    else:
        filterRedundantMatches("%s.%s_tmp" % (groupid, collect),
                                "%s.%s" % (groupid, collect) )
    if clean:
        os.remove("%s.%s_tmp" % (groupid, collect) )


def getRmvPairAlignInChunkOverlapsCmd(verbose, projectName, config, sectionNamePrepareBatches, iLauncher):
    lArgs = []
    lArgs.append("-i %s.align" % projectName)
    lArgs.append("-l %s" % config.get(sectionNamePrepareBatches, "chunk_length"))
    lArgs.append("-o %s" % config.get(sectionNamePrepareBatches, "chunk_overlap"))
    lArgs.append("-m %d" % 10)
    lArgs.append("-O %s.align.not_over" % projectName)
    lArgs.append("-v %d" % (verbose - 1))
    
    return iLauncher.getSystemCommand("%s/bin/RmvPairAlignInChunkOverlaps.py" % os.environ["REPET_PATH"], lArgs)


def getFilterAlignCmd(verbose, sectionName, config, iLauncher):
    lArgs = []
    lArgs.append("-i %s.align.not_over" % projectName)
    lArgs.append("-E %s" % config.get(sectionName,"min_Evalue"))
    lArgs.append("-I %s" % config.get(sectionName,"min_identity"))
    lArgs.append("-l %s" % config.get(sectionName,"min_length"))
    lArgs.append("-L %s" % config.get(sectionName,"max_length"))
    lArgs.append("-v %d" % (verbose - 1))
    
    return iLauncher.getSystemCommand("%s/bin/FilterAlign.py" % os.environ["REPET_PATH"], lArgs)       
        
#------------------------------------------------------------------------------

def selfAlign( smplAlign ):
    """
    step 2: self-alignment of the input data bank

    @param smplAlign: name of the alignment program (Blaster or Pals)
    @type smplAlign: string
    """

    if verbose > 0:
        print "beginning of step 2"
        print "self-alignment with %s" % ( smplAlign )
        sys.stdout.flush()

    if os.path.exists( projectName + "_" + smplAlign ):
        print "ERROR: directory " + projectName + "_" + smplAlign + "/ already exists"
        sys.exit(1)

    sectionName = "self_align"
    isCopyOnNode = False
    
    try:
        CheckerUtils.checkSectionInConfigFile(config, sectionName)
    except NoSectionError:
        print "ERROR: the section " + sectionName + " must be in your config file : " + configFileName
        sys.exit(1)

    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "blast")
    except NoOptionError:
        print "ERROR: the option blast must be define in " + sectionName +" in your config file : " + configFileName
        sys.exit(1)
    
    blastToUse = config.get(sectionName, "blast")
    
    if blastToUse not in ["ncbi", "blastplus", "wu"]:
        print "ERROR: 'blast: %s' in your configuration file is not authorized" % blastToUse
        sys.exit(1)
        
    if blastToUse == "blastplus":
        if not (CheckerUtils.isExecutableInUserPath("blastn") or CheckerUtils.isExecutableInUserPath("makeblastdb")):
            print "ERROR: ncbi blast plus (blastn executable) must be in your path"
            sys.exit(1)
        else:
            process = subprocess.Popen("blastn -h", shell = True, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
            output = process.communicate()[0]
            if "washu" in output.lower():
                print "ERROR: you are trying to use NCBI BlastPlus, but the blastn executable is pointing to WuBlast. Please check your system path."
                sys.exit(1)
    elif blastToUse == "wu":
        if not (CheckerUtils.isExecutableInUserPath("blastn") or CheckerUtils.isExecutableInUserPath("xdformat")):
            print "ERROR: WuBlast (blastn executable) must be in your path"
            sys.exit(1)
        else:
            process = subprocess.Popen("blastn -h", shell = True, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
            output = process.communicate()[0]
            if not "washu" in output.lower():
                print "ERROR: you are trying to use WuBlast, but the blastn executable is pointing to NCBI BlastPlus. Please check your system path."
                sys.exit(1)
    else:
        if not (CheckerUtils.isExecutableInUserPath("blastall") or CheckerUtils.isExecutableInUserPath("formatdb")):
            print "ERROR: ncbi blastall and formatdb must be in your path"
            sys.exit(1)
                
    if smplAlign == "Blaster":
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "Evalue") 
        except NoOptionError:
            print "ERROR: the option Evalue must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
    if smplAlign == "Pals" and not CheckerUtils.isExecutableInUserPath("pals"):
        print "ERROR: 'pals' must be in your path"
        sys.exit(1)
    if smplAlign == "Yass" and not CheckerUtils.isExecutableInUserPath("yass"):
        print "ERROR: 'yass' must be in your path"
        sys.exit(1)
    if smplAlign == "Blat" and not CheckerUtils.isExecutableInUserPath("blat"):
        print "ERROR: 'blat' must be in your path"
        sys.exit(1)

    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "clean") 
    except NoOptionError:
        print "ERROR: the option 'clean' must be define in " + sectionName +" in your config file : " + configFileName
        sys.exit(1) 
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "length") 
    except NoOptionError:
        print "ERROR: the option 'length' must be define in " + sectionName +" in your config file : " + configFileName
        sys.exit(1)
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "identity") 
    except NoOptionError:
        print "ERROR: the option 'identity' must be define in " + sectionName +" in your config file : " + configFileName
        sys.exit(1)

    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "filter_HSP") 
    except NoOptionError:
        print "ERROR: the option 'filter_HSP' must be define in " + sectionName +" in your config file : " + configFileName
        sys.exit(1)      
    if config.get(sectionName,"filter_HSP") == "yes":
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "min_Evalue") 
        except NoOptionError:
            print "ERROR: the option 'min_Evalue' must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "min_identity") 
        except NoOptionError:
            print "ERROR: the option 'min_identity' must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "min_length") 
        except NoOptionError:
            print "ERROR: the option 'min_length' must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "max_length") 
        except NoOptionError:
            print "ERROR: the option 'max_length' must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
    
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "resources") 
    except NoOptionError:
        print "ERROR: the option 'resources' must be define in " + sectionName +" in your config file : " + configFileName
        sys.exit(1)
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "tmpDir") 
    except NoOptionError:
        print "ERROR: the option 'tmpDir' must be define in " + sectionName +" in your config file : " + configFileName
        sys.exit(1)
          
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "copy")
    except NoOptionError:
        print "ERROR: the option 'copy' must be define in " + sectionName +" in your config file : " + configFileName
        sys.exit(1)
    if config.get(sectionName,"copy") == "yes" and config.get(sectionName,"tmpDir") != "":
        isCopyOnNode = True
        
    if verbose > 0:
        print "The copy option is: %s " % isCopyOnNode
        sys.stdout.flush()   
            
    os.mkdir("%s_%s" % (projectName, smplAlign))
    os.chdir("%s_%s" % (projectName, smplAlign))
    os.system( "cp ../%s ." % configFileName)
                                  
    queue = config.get(sectionName, "resources")
    cDir = os.getcwd()
    if config.get(sectionName, "tmpDir" ) != "":
        tmpDir = config.get(sectionName, "tmpDir")
    else:
        tmpDir = cDir

    # if the program is Blaster
    if smplAlign == "Blaster":

        # launch Blaster on all the chunks in parallel
        if verbose > 0:
            print "START Blaster in parallel" 
            sys.stdout.flush()
            
        groupid = "%s_TEdenovo_S2_%s" % (projectName, smplAlign)
        acronym = "Blaster"
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid, acronym, chooseTemplateWithCopy = isCopyOnNode)

        lCmdsTuples = []
        fileSize = float(os.path.getsize("%s/%s_db/%s_chunks.fa" % (projectDir, projectName, projectName)) + 5000000) / 1000000000
        
        lCmdSize = []
        lCmdCopy = []
        if isCopyOnNode:
            lCmdSize.append("fileSize = %f" % fileSize)       
            lCmdCopy.append("shutil.copy(\"%s/%s_db/%s_chunks.fa\", \".\")" % (projectDir, projectName, projectName))
            
        lFiles = FileUtils.getFileNamesList("%s/%s_db/batches" % (projectDir, projectName), "batch_")
        for file in lFiles:
            lCmds = []
            lCmds.append(getBlasterCmd(verbose, projectName, sectionName, config, iLauncher, file))
            lCmdStart = []
            if isCopyOnNode:
                lCmdStart.append("os.symlink(\"../%s_chunks.fa\", \"%s_chunks.fa\")" % (projectName, projectName))
                lCmdStart.append("shutil.copy(\"%s/%s_db/batches/%s\", \".\")" % (projectDir, projectName, file))
            else:
                lCmdStart.append("os.symlink(\"%s/%s_db/%s_chunks.fa\", \"%s_chunks.fa\")" % (projectDir, projectName, projectName, projectName))
                lCmdStart.append("os.symlink(\"%s/%s_db/batches/%s\", \"%s\")" % (projectDir, projectName, file, file))
            lCmdFinish = []
            lCmdFinish.append("if os.path.exists(\"%s.align\"):" % file)
            lCmdFinish.append("\tshutil.move(\"%s.align\", \"%s/.\" )" % (file, cDir))
            lCmdFinish.append("shutil.move(\"%s.param\", \"%s/.\" )" % (file, cDir))
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish, lCmdSize, lCmdCopy))
        
        cleanMustBeDone = False
        if config.get(sectionName, "clean") == "yes":
            cleanMustBeDone = True
        iLauncher.runLauncherForMultipleJobs("Blaster", lCmdsTuples, cleanMustBeDone, isCopyOnNode)
        runCollect(groupid, "align", True, cleanMustBeDone)
        
        if verbose > 0:
            print "END Blaster in parallel" 
            sys.stdout.flush()
        
        log = os.system("mv %s.align %s.align" % (groupid, projectName) )
        if log != 0:
            print "ERROR while renaming the 'align' file"
            sys.exit(1)

        # clean 'cut' files
#        if config.get(sectionName,"clean") == "yes":
#            os.system( "rm -f formatdb.log %s_chunks.fa_cut*" % ( projectName ) )

    # if the program is Pals
    #TODO: to test...
    elif smplAlign == "Pals":
        
        # launch Pals on all the chunks in parallel
        prg = os.environ["REPET_PATH"] + "/bin/srptPals.py"
        cmd = prg
        cmd += " -t jobs"
        cmd += " -g %s_TEdenovo_%s_srptPals" % ( projectName, smplAlign )
        cmd += " -q %s/%s_db/batches" % ( projectDir, projectName )
        cmd += " -s %s/%s_db/%s_chunks.fa" % (projectDir, projectName, projectName)
        cmd += " -Q \"%s\"" % config.get(sectionName,"resources")
        if config.get(sectionName,"tmpDir" ) != "":
            cmd += " -d %s" % ( config.get(sectionName,"tmpDir") )
        cmd += " -C %s" % ( configFileName )
        cmd += " -P \""
        cmd += "-length " + config.get(sectionName,"length")  #400
        cmd += " -pctid " + config.get(sectionName,"identity")  #94
        cmd += "\""
        cmd += " -Z"
        cmd += " -a"
        if config.get(sectionName,"clean") == "yes":
            cmd += " -c"
        log = os.system( cmd )
        if log != 0:
            print "ERROR: %s returned %i" % ( prg, log )
            sys.exit(1)

        os.system( "mv %s_TEdenovo_%s_srptPals.align %s.align" % ( projectName, smplAlign, projectName ) )

    # if the program is Yass or Blat or Blast
    #TODO: to test...
    elif smplAlign in [ "Yass", "Blat", "Blast" ]:
        prg = os.environ["REPET_PATH"] + "/bin/%sClusterLauncher.py" % ( smplAlign )
        cmd = prg
        cmd += " -i %s/%s_db/batches" % ( projectDir, projectName )
        cmd += " -Q \"%s\"" % ( config.get(sectionName,"resources") )
        cmd += " -g %s_TEdenovo_%s_%sClusterLauncher" % ( projectName, smplAlign, smplAlign )
        cmd += " -C %s" % ( configFileName )
        if config.get(sectionName,"tmpDir") != "":
            cmd += " -d %s" % ( config.get(sectionName,"tmpDir") )
        cmd += " -j jobs"
        cmd += " -s %s/%s_db/%s_chunks.fa" % (projectDir, projectName, projectName)
        if smplAlign == "Yass":
            cmd += " -p \"-E %s\"" % ( config.get(sectionName,"Evalue") )
        elif smplAlign == "Blat":
            cmd += " -p \"-minIdentity=%s\"" % ( config.get(sectionName,"identity") )
        elif smplAlign == "Blast":
            cmd += " -p blastn -P \"-F F -e %s\"" % ( config.get(sectionName,"Evalue") )
        cmd += " -Z"
        cmd += " -A"
        if config.get(sectionName,"clean") == "yes":
            cmd += " -c"
        cmd += " -v %i" % ( verbose - 1 )
        log = os.system( cmd )
        if log != 0:
            print "ERROR: %s returned %i" % ( prg, log )
            sys.exit(1)
            
        os.system( "mv %s_TEdenovo_%sClusterLauncher.align %s.align" % ( projectName, smplAlign, projectName ) )

    groupid = "%s_TEdenovo_%s_RmvPairAlignInChunkOverlaps" % ( projectName, smplAlign )
    iDb = DbFactory.createInstance()
    jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
    iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)

    lCmdsTuples = []
    sectionNamePrepareBatches = "prepare_batches"
    # remove the chunk overlaps
    lCmds = []
    lCmds.append(getRmvPairAlignInChunkOverlapsCmd(verbose, projectName, config, sectionNamePrepareBatches, iLauncher))
    lCmdStart = [] 
    lCmdStart.append("os.symlink(\"%s/%s.align\", \"%s.align\")" % (cDir, projectName, projectName))
    lCmdFinish = []
    lCmdFinish.append("shutil.move(\"%s.align.not_over\", \"%s/.\")" % (projectName, cDir))
    lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))

    cleanMustBeDone = False
    if config.get(sectionName, "clean") == "yes":
        cleanMustBeDone = True
    iLauncher.runLauncherForMultipleJobs("RmvPairAlignInChunkOverlaps", lCmdsTuples, cleanMustBeDone)
    
    # filter the HSPs
    
    if config.get(sectionName,"filter_HSP") == "yes":
        groupid = "%s_TEdenovo_%s_FilterAlign" % ( projectName, smplAlign )
        acronym = "FilterAlign"
        iDb = DbFactory.createInstance()
        jobdb = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
        
        lCmdsTuples = []
        lCmds = []    
        lCmds.append(getFilterAlignCmd(verbose, sectionName, config, iLauncher))
        lCmdStart = []
        lCmdStart.append("os.symlink(\"%s/%s.align.not_over\", \"%s.align.not_over\")" % (cDir, projectName, projectName))
        lCmdFinish = []
        lCmdFinish.append("shutil.move(\"%s.align.not_over.filtered\", \"%s/.\")" % (projectName, cDir))
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
    
        cleanMustBeDone = False
        if config.get(sectionName, "clean") == "yes":
            cleanMustBeDone = True
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)    
    
    # clean
    if config.get(sectionName,"clean") == "yes":

        # for each job, remove the file containing the HSPs
        cmd = "find . -name \"batch_*.align\" -exec rm {} \;"
        log = os.system( cmd )
        if log != 0:
            print "ERROR while removing the 'align' files from the jobs"
            sys.exit(1)

        # remove the temporary file(s) with the HSPs
        if config.get(sectionName,"filter_HSP") == "yes":
            cmd = "rm -f %s.align %s.align.not_over" % ( projectName, projectName )
            log = os.system( cmd )
            if log != 0:
                print "ERROR while removing an intermediary file"
                sys.exit(1)
                
        os.chdir("%s/%s_db" % (projectDir, projectName))
        # remove the files "_cut"
        cmd = "find . -name \"%s*_cut*\" -exec rm {} \;" % ( projectName )
        log = os.system( cmd )
        if log != 0:
            print "ERROR while removing the '_cut' files"
            sys.exit(1)

        # remove the files "Nstretch.map"
        cmd = "find . -name \"*Nstretch.map\" -exec rm {} \;"
        log = os.system( cmd )
        if log != 0:
            print "ERROR while removing the 'Nstretch.map' files"
            sys.exit(1)

    os.chdir( ".." )

    if verbose > 0:
        print "step 2 finished successfully"
        sys.stdout.flush()
    
#------------------------------------------------------------------------------

def getGrouperCmd(verbose, sectionNameClusterHSP, config, alignFileName, bankName, coverage, iLauncher):
    lArgs = []
    lArgs.append("-q %s" % bankName)
    lArgs.append("-s %s" % bankName)
    lArgs.append("-m %s" % alignFileName)
    if config.get(sectionNameClusterHSP, "Grouper_join") == "yes":
        lArgs.append("-j")
    lArgs.append("-C %s" % coverage)
    lArgs.append("-Z %s" % config.get(sectionNameClusterHSP, "minNbSeqPerGroup"))
    lArgs.append("-X %s" % config.get(sectionNameClusterHSP, "Grouper_include"))
    lArgs.append("-G -1")
    lArgs.append("-v %i" % verbose)
    return iLauncher.getSystemCommand("%s/bin/grouper" % os.environ["REPET_PATH"], lArgs) 
    
def getFilterOutGrouperCmd(smplAlign, clustHsp, verbose, sectionNameClusterHSP, sectionNamePrepareBatches, config, projectName, alignFileName, minSeq, maxSeq, iLauncher):
    lArgs = []
    lArgs.append("-i %s.group.c%s.fa" % (alignFileName, config.get(sectionNameClusterHSP, "Grouper_coverage")))
    lArgs.append("-m %s" % minSeq)
    lArgs.append("-M %s" % maxSeq)
    lArgs.append("-L %s" % (config.get(sectionNameClusterHSP, "maxSeqLength")))
    lArgs.append("-J %s" % (config.get(sectionNameClusterHSP, "Grouper_maxJoinLength")))
    if "onChr" not in alignFileName:
        lArgs.append("-O %s" % config.get(sectionNamePrepareBatches, "chunk_overlap"))
    else:
        lArgs.append("-O -1")
    lArgs.append("-o %s_%s_%s_%selem_%sseq.fa" % (projectName, smplAlign, clustHsp, minSeq, maxSeq))
    lArgs.append("-v %i" % (verbose - 1))    
    return iLauncher.getSystemCommand("%s/bin/filterOutGrouper.py" % os.environ["REPET_PATH"], lArgs) 

def getLaunchReconOrPilerCmd(smplAlign, clustHsp, verbose, sectionNameClusterHSP, config, projectName, alignFileName, bankName, iLauncher):
    lArgs = []
    lArgs.append("-p %s_%s" % (projectName, smplAlign))
    lArgs.append("-f %s" % bankName)
    lArgs.append("-a %s" % alignFileName)
    if config.get(sectionNameClusterHSP, "clean") == "yes":
        lArgs.append("-c")
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand("%s/bin/launch%s.py" % (os.environ["REPET_PATH"], clustHsp), lArgs)     

def getFilterSeqClusterCmd(dirName, clustHsp, minSeq, maxSeq, sectionNameClusterHSP, verbose, iLauncher):
    lArgs = []
    lArgs.append("-i %s.map" % dirName)
    lArgs.append("-c %s" % clustHsp)
    lArgs.append("-m %s" % minSeq)
    lArgs.append("-M %s" % maxSeq)
    lArgs.append("-L %s" % config.get(sectionNameClusterHSP, "maxSeqLength"))
    lArgs.append("-d")
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand("%s/bin/FilterSeqClusters.py" % os.environ["REPET_PATH"], lArgs)  

def getMap2dbCmd(dirName, bankName, minSeq, maxSeq, iLauncher):
    lArgs = []
    lArgs.append("%s.map.filtered-%s-%s" % (dirName, minSeq, maxSeq))
    lArgs.append(bankName)
    return iLauncher.getSystemCommand("%s/bin/map2db" % os.environ["REPET_PATH"], lArgs) 
    
#------------------------------------------------------------------------------

def clusterHsp( smplAlign, clustHsp ):
    """
    step 3: cluster the resulting HSPs (high-scoring segment pairs)

    @param smplAlign: name of the alignment program (Blaster or Pals)
    @type smplAlign: string

    @param clustHsp: name of the clustering program (Grouper, Recon or Piler)
    @type clustHsp: string
    """

    if verbose > 0:
        print "beginning of step 3"
        print "self-alignment with %s" % smplAlign
        print "clustering with %s" % clustHsp
        sys.stdout.flush()

    selfAlignSectionName = "self_align"
    sectionName = "cluster_HSPs"
    try:
        CheckerUtils.checkSectionInConfigFile(config, sectionName)
    except NoSectionError:
        print "ERROR: '%s' must be in your config file : %s" % (sectionName, configFileName)
        sys.exit(1)

    if clustHsp == "Grouper" or clustHsp == "Recon" or clustHsp == "Piler":
        dirName = "%s_%s_%s" % (projectName, smplAlign, clustHsp)
        if os.path.exists(dirName):
            print "ERROR: directory %s already exists" % dirName
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "minNbSeqPerGroup") 
        except NoOptionError:
            print "ERROR: the option minNbSeqPerGroup must be define in '%s' in your config file : %s" % (sectionName, configFileName)
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "nbLongestSeqPerGroup") 
        except NoOptionError:
            print "ERROR: the option nbLongestSeqPerGroup must be define in '%s' in your config file : %s" % (sectionName, configFileName)
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, selfAlignSectionName, "filter_HSP") 
        except NoOptionError:
            print "ERROR: the option filter_HSP must be define in '%s' in your config file : %s" % (selfAlignSectionName, configFileName)
            sys.exit(1)
            
        if int(config.get(sectionName, "minNbSeqPerGroup")) < 3:
            print "ERROR: groups with less than %i members should be filtered out" % int(config.get(sectionName, "minNbSeqPerGroup"))
            print "as they are likely to correspond to segmental duplications"
            print "change this parameter value in your configuration file"
            sys.exit(1)

        os.mkdir(dirName)
        os.chdir(dirName)

        alignFileName = "%s.align.not_over" % projectName
        if config.get(selfAlignSectionName, "filter_HSP") == "yes":
            alignFileName += ".filtered"
        pathToAlignFile = "%s/%s_%s/%s" % (projectDir, projectName, smplAlign, alignFileName)
        if os.path.exists("%s.onChr" % pathToAlignFile):
            alignFileName += ".onChr"
            pathToAlignFile += ".onChr"

        sectionNamePrepareBatches = "prepare_batches"
        bankName = "%s_chunks.fa" % projectName
        if "onChr" in alignFileName:
            bankName = "%s.fa" % projectName
            try:
                CheckerUtils.checkOptionInSectionInConfigFile(config, sectionNamePrepareBatches, "chunk_overlap")
            except NoOptionError:
                print "ERROR: the option chunk_overlap must be define in '%s' in your config file" % (sectionNamePrepareBatches)
                sys.exit(1)
        pathToBank = "%s/%s_db/%s" % (projectDir, projectName, bankName)

        #----------------------------------------------------------------------

        lCmds = []
        minSeq = config.get(sectionName, "minNbSeqPerGroup")
        maxSeq = config.get(sectionName, "nbLongestSeqPerGroup")

        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "clean") 
        except NoOptionError:
            print "ERROR: the option 'clean' must be define in '%s' in your config file : %s" % (sectionName, configFileName)
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "maxSeqLength") 
        except NoOptionError:
            print "ERROR: the option 'maxSeqLength' must be define in '%s' in your config file :  %s" % (sectionName, configFileName)
            sys.exit(1)
                
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "resources") 
        except NoOptionError:
            print "ERROR: the option 'resources' must be define in '%s' in your config file :  %s" % (sectionName, configFileName)
            sys.exit(1)
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "tmpDir") 
        except NoOptionError:
            print "ERROR: the option 'tmpDir' must be define in '%s' in your config file :  %s" % (sectionName, configFileName)
            sys.exit(1)
        os.symlink("../%s" % configFileName, configFileName)
        queue = config.get(sectionName, "resources")
        cDir = os.getcwd()
        if config.get(sectionName, "tmpDir") != "":
            tmpDir = config.get(sectionName, "tmpDir")
        else:
            tmpDir = cDir
            
        groupid = "%s_TEdenovo_%s_%s" % (projectName, smplAlign, clustHsp)
        acronym = clustHsp
        jobdb = RepetJob()
        
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
        lCmdsTuples = []
        # remove the chunk overlaps
        lCmds = []
        lCmdStart = [] 
        lCmdStart.append("os.symlink(\"%s\", \"%s\")" % (pathToBank, bankName))
        lCmdStart.append("os.symlink(\"%s\", \"%s\")" % (pathToAlignFile, alignFileName))
        lCmdFinish = []

        if clustHsp == "Grouper":
            try:
                CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "Grouper_coverage") 
            except NoOptionError:
                print "ERROR: the option Grouper_coverage must be define in '%s' in your config file : %s" % (sectionName, configFileName)
                sys.exit(1)
            try:
                CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "Grouper_join") 
            except NoOptionError:
                print "ERROR: the option Grouper_join must be define in '%s' in your config file : %s" % (sectionName, configFileName)
                sys.exit(1)
            try:
                CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "Grouper_include") 
            except NoOptionError:
                print "ERROR: the option Grouper_include must be define in '%s' in your config file : %s" % (sectionName, configFileName)
                sys.exit(1)
            try:
                CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, "Grouper_maxJoinLength") 
            except NoOptionError:
                print "ERROR: the option Grouper_maxJoinLength must be define in '%s' in your config file : %s" % (sectionName, configFileName)
                sys.exit(1)
                
            coverage = config.get(sectionName, "Grouper_coverage")
            cmd = getGrouperCmd(verbose, sectionName, config, alignFileName, bankName, coverage, iLauncher)
            lCmds.append(cmd)
            lCmdFinish.append("shutil.move(\"%s.group.c%s.fa\", \"%s/.\")" % (alignFileName, coverage, cDir))
            lCmdFinish.append("shutil.move(\"%s.group.c%s.param\", \"%s/.\")" % (alignFileName, coverage, cDir))
            lCmdFinish.append("shutil.move(\"%s.group.c%s.set\", \"%s/.\")" % (alignFileName, coverage, cDir))
            lCmdFinish.append("shutil.move(\"%s.group.c%s.txt\", \"%s/.\")" % (alignFileName, coverage, cDir))
            lCmdFinish.append("shutil.move(\"%s.chains.path\",  \"%s/.\")" % (alignFileName, cDir))
            cmd = getFilterOutGrouperCmd(smplAlign, clustHsp, verbose, sectionName, sectionNamePrepareBatches, config, projectName, alignFileName, minSeq, maxSeq, iLauncher)
            lCmds.append(cmd)
            lCmdFinish.append("shutil.move(\"%s_%s_%s_%selem_%sseq.fa\", \"%s\")" % (projectName, smplAlign, clustHsp, minSeq, maxSeq, cDir))
            lCmdFinish.append("shutil.move(\"%s.group.c%s.fa_filtered.log\", \"%s\")" % (alignFileName, config.get(sectionName, "Grouper_coverage"), cDir))
            
        else:
            if clustHsp == "Recon":
                if not CheckerUtils.isExecutableInUserPath("recon.pl") :
                    print "ERROR: recon.pl must be in your path"
                    sys.exit(1)
                lCmdFinish.append("shutil.move(\"%s_%s_Recon.map\", \"%s/.\")" % (projectName, smplAlign, cDir))
                lCmdFinish.append("shutil.move(\"%s_%s_Recon.fa\", \"%s/.\")" % (projectName, smplAlign, cDir))
                lCmdFinish.append("shutil.move(\"%s_%s_MSP_file\", \"%s/.\")" % (projectName, smplAlign, cDir))
                lCmdFinish.append("shutil.move(\"%s_%s_seq_list\", \"%s/.\")" % (projectName, smplAlign, cDir))
                if config.get(sectionName, "clean") == "no":
                    lCmdFinish.append("shutil.move(\"images\", \"%s/images\")" % cDir)
                    lCmdFinish.append("shutil.move(\"ele_def_res\", \"%s/ele_def_res\")" % cDir)
                    lCmdFinish.append("shutil.move(\"ele_redef_res\", \"%s/ele_redef_res\")" % cDir)
                    lCmdFinish.append("shutil.move(\"edge_redef_res\", \"%s/edge_redef_res\")" % cDir)
                    lCmdFinish.append("shutil.move(\"summary\", \"%s/summary\")" % cDir)
                lCmdFinish.append("shutil.move(\"%s.map.filtered-%s-%s\", \"%s/.\")" % (dirName, minSeq, maxSeq, cDir))
                lCmdFinish.append("shutil.move(\"%s.map_filtered.log\", \"%s/.\")" % (dirName, cDir))
                lCmdFinish.append("shutil.move(\"%s.map.distribC\", \"%s/.\")" % (dirName, cDir))
                lCmdFinish.append("shutil.move(\"%s.map.filtered-%s-%s.flank_size0.fa\", \"%s/.\")" % (dirName, minSeq, maxSeq, cDir))
            
            elif clustHsp == "Piler":
                if not CheckerUtils.isExecutableInUserPath("piler") :
                    print "ERROR: piler must be in your path"
                    sys.exit(1)
                lCmdFinish.append("shutil.move(\"%s_%s_Piler.map\", \"%s/.\")" % (projectName, smplAlign, cDir))
                lCmdFinish.append("shutil.move(\"%s_%s_Piler.fa\", \"%s/.\")" % (projectName, smplAlign, cDir))
                lCmdFinish.append("shutil.move(\"%s.gff\", \"%s/.\")" % (alignFileName, cDir))
                lCmdFinish.append("shutil.move(\"%s_%s_Piler-trs.gff\", \"%s/.\")" % (projectName, smplAlign, cDir))
                lCmdFinish.append("shutil.move(\"%s.map.filtered-%s-%s\", \"%s/.\")" % (dirName, minSeq, maxSeq, cDir))
                lCmdFinish.append("shutil.move(\"%s.map_filtered.log\", \"%s/.\")" % (dirName, cDir))
                lCmdFinish.append("shutil.move(\"%s.map.distribC\", \"%s/.\")" % (dirName, cDir))
                lCmdFinish.append("shutil.move(\"%s.map.filtered-%s-%s.flank_size0.fa\", \"%s/.\")" % (dirName, minSeq, maxSeq, cDir))
            
            cmd = getLaunchReconOrPilerCmd(smplAlign, clustHsp, verbose, sectionName, config, projectName, alignFileName, bankName, iLauncher)
            lCmds.append(cmd)
            cmd = getFilterSeqClusterCmd(dirName, clustHsp, minSeq, maxSeq, sectionName, verbose, iLauncher)
            lCmds.append(cmd)
            cmd = getMap2dbCmd(dirName, bankName, minSeq, maxSeq, iLauncher)
            lCmds.append(cmd)
        
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        
        cleanMustBeDone = False
        if config.get(sectionName, "clean") == "yes":
            cleanMustBeDone = True
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)
        
        if clustHsp in ["Recon", "Piler"]:
            os.symlink("%s.map.filtered-%s-%s.flank_size0.fa" % (dirName, minSeq, maxSeq), "%s_%selem_%sseq.fa" % (dirName, minSeq, maxSeq))

        os.chdir( ".." )

    #--------------------------------------------------------------------------

    elif clustHsp == "":
        print "ERROR: missing option -c"
        help()
        sys.exit(1)

    else:
        print "ERROR: unknown option -c %s" % clustHsp
        sys.exit(1)

    if verbose > 0:
        print "step 3 finished successfully"
        sys.stdout.flush()

#------------------------------------------------------------------------------

def getSplitSeqPerClusterCmd(prefix2, clustHsp, verbose, iLauncher):
    lArgs = []
    lArgs.append("-i %s.fa" % prefix2)
    lArgs.append("-c %s" % clustHsp)
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand("%s/bin/splitSeqPerCluster.py" % os.environ["REPET_PATH"], lArgs)

def getConvertFastaHeadersFromChkToChrCmd(file, mapChunkFile, iLauncher):
    lArgs = []
    lArgs.append("-i %s" % file)
    lArgs.append("-m %s" % mapChunkFile)
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand("%s/bin/ConvertFastaHeadersFromChkToChr.py" % os.environ["REPET_PATH"], lArgs)

def getMapProgramLauncherCmd(inFile, sectionNameBuildConsensus, iLauncher):
    lArgs = []
    lArgs.append("-i %s" % inFile)
    if config.get(sectionNameBuildConsensus, "clean") == "yes":
        lArgs.append("-c")
    lArgs.append("-v %i" % verbose)
    return iLauncher.getSystemCommand("%s/bin/MapProgramLauncher.py" % os.environ["REPET_PATH"], lArgs)

def getMafftProgramLauncherCmd(inFile, sectionNameBuildConsensus, iLauncher):
    lArgs = []
    lArgs.append("-i %s" % inFile)
    if config.get(sectionNameBuildConsensus, "clean") == "yes":
        lArgs.append("-c")
    lArgs.append("-v %i" % verbose)
    return iLauncher.getSystemCommand("%s/bin/MafftProgramLauncher.py" % os.environ["REPET_PATH"], lArgs)

def getBuildConsLibCmd(sectionNameBuildConsensus, prefix, multAlign, iLauncher):
    lArgs = []
    lArgs.append("-d %s" % os.getcwd())
    lArgs.append("-r \'*.fa_aln\'")
    lArgs.append("-n %s" % config.get(sectionNameBuildConsensus, "minBasesPerSite"))
    lArgs.append("-o %s_%s_consensus.fa" % (prefix, multAlign))
    if config.get(sectionNameBuildConsensus, "clean") == "yes":
        lArgs.append("-c")
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand("%s/bin/buildConsLib.py" % os.environ["REPET_PATH"], lArgs)
    
#------------------------------------------------------------------------------

def buildConsensus( smplAlign, clustHsp, multAlign, isStructuralSearch ):

    """
    step 4: build consensus from multiple alignments

    @param smplAlign: name of the alignment program (Blaster or Pals)
    @type smplAlign: string

    @param clustHsp: name of the clustering program (Grouper, Recon or Piler)
    @type clustHsp: string

    @param multAlign: name of the multiple alignment program MSA (MAP, Mafft, Muscle, Tcoffee, ClustalW or Prank)
    @type multAlign: string
    """
    
    if isStructuralSearch:
        smplAlign = "LTRharvest"
        clustHsp = "Blastclust"
        
    if verbose > 0:
        print "beginning of step 4"
        print "multiple alignment with %s" % multAlign
        sys.stdout.flush()

    isOptionErrorRaised = False

    sectionNameClusterHSPs = "cluster_HSPs"
    try:
        CheckerUtils.checkSectionInConfigFile(config, sectionNameClusterHSPs) 
    except NoSectionError:
        print "ERROR: the section '%s' must be define in your config file : %s" % (sectionNameClusterHSPs, configFileName)
        sys.exit(1)
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionNameClusterHSPs, "minNbSeqPerGroup") 
    except NoOptionError:
        print "ERROR: the option 'minNbSeqPerGroup' must be define in '%s' in your config file : %s" % (sectionNameClusterHSPs, configFileName)
        isOptionErrorRaised = True
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionNameClusterHSPs, "nbLongestSeqPerGroup") 
    except NoOptionError:
        print "ERROR: the option 'nbLongestSeqPerGroup' must be define in '%s' in your config file : %s" % (sectionNameClusterHSPs, configFileName)
        isOptionErrorRaised = True
   
    sectionNameBuildConsensus = "build_consensus" 
    try:
        CheckerUtils.checkSectionInConfigFile(config, sectionNameBuildConsensus) 
    except NoSectionError:
        print "ERROR: the section '%s' must be define in your config file : %s" % (sectionNameBuildConsensus, configFileName)
        sys.exit(1)
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionNameBuildConsensus, "resources") 
    except NoOptionError:
        print "ERROR: the option 'resources' must be define in '%s' in your config file : %s" % (sectionNameBuildConsensus, configFileName)
        isOptionErrorRaised = True
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionNameBuildConsensus, "tmpDir") 
    except NoOptionError:
        print "ERROR: the option 'tmpDir' must be define in '%s' in your config file : %s" % (sectionNameBuildConsensus, configFileName)
        isOptionErrorRaised = True
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionNameBuildConsensus, "minBasesPerSite") 
    except NoOptionError:
        print "ERROR: the option 'minBasesPerSite' must be define in '%s' in your config file : %s" % (sectionNameBuildConsensus, configFileName)
        isOptionErrorRaised = True
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionNameBuildConsensus, "clean") 
    except NoOptionError:
        print "ERROR: the option 'clean' must be define in  in your config file : %s" % (sectionNameBuildConsensus, configFileName)
        isOptionErrorRaised = True
    
    if isOptionErrorRaised:
        sys.exit(1)
        
    prefix1 = "%s_%s_%s" % (projectName, smplAlign, clustHsp)
    prefix2 = prefix1
    if clustHsp == "Blastclust":
        prefix2 += "_1elem_20seq"
    else:
        prefix2 += "_%selem_%sseq" % (config.get(sectionNameClusterHSPs, "minNbSeqPerGroup"), config.get(sectionNameClusterHSPs, "nbLongestSeqPerGroup"))

    if clustHsp == "Grouper" or clustHsp == "Recon" or clustHsp == "Piler" or clustHsp == "Blastclust":

        if not os.path.exists( "%s/%s.fa" % ( prefix1, prefix2 ) ):
            print "ERROR: %s/%s.fa not found" % ( prefix1, prefix2 )
            sys.exit(1)

        if os.path.exists( "%s_%s" % ( prefix1, multAlign ) ):
            print "ERROR: directory %s_%s/ already exists" % ( prefix1, multAlign )
            sys.exit(1)

        if multAlign not in ["Map","MAP","Mafft","Clustalw","Muscle","Prank","Tcoffee"]:
            print "ERROR: %s not implememented" % ( multAlign )
            sys.exit(1)

        os.mkdir( "%s_%s" % ( prefix1, multAlign ) )
        os.chdir( "%s_%s" % ( prefix1, multAlign ) )
        
        os.system( "ln -s ../%s ." % ( configFileName ) )
        queue = config.get(sectionNameBuildConsensus, "resources")
        cDir = os.getcwd()
        if config.get(sectionNameBuildConsensus, "tmpDir" ) != "":
            tmpDir = config.get(sectionNameBuildConsensus, "tmpDir")
        else:
            tmpDir = cDir
            
        cleanMustBeDone = False
        if config.get(sectionNameBuildConsensus, "clean") == "yes":
            cleanMustBeDone = True
            
        groupid = "%s_TEdenovo_S4_%s_splitSeqPerCluster" % (projectName, clustHsp)
        jobdb = RepetJob()
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
        lCmds = []
        lCmds.append(getSplitSeqPerClusterCmd(prefix2, clustHsp, verbose, iLauncher))
        lCmdStart = []
        lCmdStart.append("os.symlink(\"%s/%s/%s.fa\", \"%s.fa\")" % (projectDir, prefix1, prefix2, prefix2))
        lCmdFinish = []
        lCmdFinish.append("os.system(\"find . -type f -name 'seqCluster*.fa' -exec mv {} %s/. \\;\")\n" % cDir)
        lCmdsTuples = []
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs("splitSeqPerCluster", lCmdsTuples, cleanMustBeDone)

        # Convert FASTA headers
        mapChunkFile = "%s/%s_db/%s_chunks.map" % (projectDir, projectName, projectName)
        lFiles = glob.glob("seqCluster*.fa")
        if os.path.exists(mapChunkFile) and lFiles != []:
            lCmdsTuples = []
            groupid = "%s_TEdenovo_S4_%s_convertHeadersFromChkToChr" % (projectName, clustHsp)
            acronym = "convertFastaHeadersFromChkToChr"
            jobdb = RepetJob()
            iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
            for file in lFiles:
                f = open(file)
                line = f.readline()
                f.close()
                if "chunk" in line:
                    f.close()
                    lCmds = []
                    lCmds.append(getConvertFastaHeadersFromChkToChrCmd(file, mapChunkFile, iLauncher))
                    lCmdStart = []
                    lCmdStart.append("os.symlink(\"%s/%s\", \"%s\")" % (cDir, file, file))
                    lCmdFinish = []
                    lCmdFinish.append("if os.path.exists(\"%s.onChr\"):" % file)
                    lCmdFinish.append("\tshutil.move(\"%s/%s\", \"%s/%s.onChk\")" % (cDir, file, cDir, file))
                    lCmdFinish.append("\tshutil.move(\"%s.onChr\", \"%s/%s\")" % (file, cDir, file))
                    lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
            iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)

        if multAlign in [ "Map", "MAP" ] and not CheckerUtils.isExecutableInUserPath("rpt_map") :
            print "ERROR: Map must be in your path"
            sys.exit(1)
        if multAlign == "Mafft" and not CheckerUtils.isExecutableInUserPath("mafft") :
            print "ERROR: mafft must be in your path"
            sys.exit(1)
        #TODO: remove this condition as long as these programs aren't used anymore and are not tested too?
        if multAlign == "Clustalw" and not CheckerUtils.isExecutableInUserPath("clustalw") :
            print "ERROR: clustalw must be in your path"
            sys.exit(1)
        if multAlign == "Muscle" and not CheckerUtils.isExecutableInUserPath("muscle") :
            print "ERROR: muscle must be in your path"
            sys.exit(1)       
        if multAlign == "Tcoffee" and not CheckerUtils.isExecutableInUserPath("t_coffee") :
            print "ERROR: t_coffee must be in your path"
            sys.exit(1)
        if multAlign == "Prank" and not CheckerUtils.isExecutableInUserPath("prank") :
            print "ERROR: prank must be in your path"
            sys.exit(1) 
        if multAlign in [ "Clustalw", "Muscle", "Tcoffee", "Prank" ]:
            prg = os.environ["REPET_PATH"] + "/bin/srpt%s.py" % ( multAlign )
            cmd = prg
            cmd += " -g %s_TEdenovo_%s_%s_%s" % (projectName, smplAlign, clustHsp, multAlign)
            cmd += " -q %s" % os.getcwd()
            cmd += " -Q \"%s\"" % config.get(sectionNameBuildConsensus, "resources")
            if config.get(sectionNameBuildConsensus, "tmpDir" ) != "":
                cmd += " -d %s" % config.get(sectionNameBuildConsensus, "tmpDir")
            if config.get(sectionNameBuildConsensus, "clean") == "yes":
                cmd += " -c"
            if multAlign == "Prank":
                cmd += " -P \"%s\"" % ( "-f=8 -noxml -notree -F -quiet" )
            elif multAlign == "Tcoffee":
                cmd += " -P \"%s\"" % ( "-type dna -output fasta_aln" )
            pL.launch( prg, cmd )

        elif multAlign in [ "Map", "MAP", "Mafft" ]:
            
            lFiles = glob.glob("seqCluster*.fa")
            inFileDirectory = os.getcwd()
            
            groupid = "%s_TEdenovo_%s_%s_%s" % (projectName, smplAlign, clustHsp, multAlign)
            acronym = multAlign
            iDb = DbFactory.createInstance()
            iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
            iLauncher = Launcher(iTJA, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
            lCmdsTuples = []
            
            for file in lFiles:
                lCmds = []
                if multAlign in [ "Map", "MAP" ]:
                    lCmds = [getMapProgramLauncherCmd(file, sectionNameBuildConsensus, iLauncher)]
                elif multAlign == "Mafft":
                    #TODO: test Mafft in functional test
                    lCmds = [getMafftProgramLauncherCmd(file, iLauncher)]
                lCmdStart = []
                lCmdStart.append("os.symlink(\"%s/%s\", \"%s\")" % (inFileDirectory, file, file))
                lCmdFinish = []
                lCmdFinish.append("shutil.copy(\"%s.fa_aln\", \"%s/.\")" % (file, inFileDirectory))
                lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
            iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)  
                
        # build the consensus for each MSA and concatenate them into a single fasta file
        groupid = "%s_TEdenovo_S4_%s_buildConsLib" % (projectName, clustHsp)
        jobdb = RepetJob()
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
        lCmds = []
        lCmds.append(getBuildConsLibCmd(sectionNameBuildConsensus, prefix1, multAlign, iLauncher))
        
        lFiles = glob.glob(cDir + "/*.fa_aln")

        lCmdStart = []
        for file in lFiles:
            lCmdStart.append("os.symlink(\"%s\", os.getcwd() + \"/%s\")" % (file, os.path.basename(file)))
        lCmdsTuples = []
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart))
        iLauncher.runLauncherForMultipleJobs("BuildConsLib_%s" % multAlign, lCmdsTuples, cleanMustBeDone)

        os.chdir("..")

    else:
        print "ERROR: %s not implememented" % clustHsp
        sys.exit(1)

    if config.get(sectionNameBuildConsensus, "clean") == "yes":
        cmd = "find . -name \"profil*\" -exec rm {} \;"
        pL.launch("find", cmd)

    if verbose > 0:
        print "step 4 finished successfully"
        sys.stdout.flush()

#------------------------------------------------------------------------------

def getShortenHeaderConsensusCmd(projectName, smplAlign, clustHSP, multAlign, isStructuralSearch, iLauncher):
    
    if isStructuralSearch:
        inFile = "%s/%s_LTRharvest_%s_%s/%s_LTRharvest_%s_%s_consensus.fa" % (projectDir, projectName, clustHSP, multAlign, projectName, clustHSP, multAlign)
        outFile = "%s_LTRharvest_%s_%s_consensus_shortH.fa" % (projectName, clustHSP, multAlign)
    else:
        inFile = "%s/%s_%s_%s_%s/%s_%s_%s_%s_consensus.fa" % (projectDir, projectName, smplAlign, clustHSP, multAlign, projectName, smplAlign, clustHSP, multAlign)
        outFile = "%s_%s_%s_%s_consensus_shortH.fa" % (projectName, smplAlign, clustHSP, multAlign)
    
    if not os.path.exists( inFile ):
        msg = "ERROR: consensus file '%s' doesn't exist" % (inFile.replace("%s/" % projectDir, ""))
        sys.stderr.write( "%s\n" % msg )
        sys.exit(1)
        
    lArgs = []
    lArgs.append("-p %s" % projectName)
    lArgs.append("-s %s" % smplAlign)
    lArgs.append("-c %s" % clustHSP)
    lArgs.append("-i %s" % inFile)
    lArgs.append("-o %s" % outFile)
    lArgs.append("-m %s" % multAlign)
    lArgs.append("-v %i" % (verbose - 1))
 
    return iLauncher.getSystemCommand("%s/bin/shortenHeaderConsensus.py" % os.environ["REPET_PATH"], lArgs)

#------------------------------------------------------------------------------

def detectFeatures( smplAlign, clustHsp, multAlign, isStructuralSearch ):

    """
    step 5: detect TE features on the consensus

    @param smplAlign: name of the alignment program (Blaster or Pals)
    @type smplAlign: string

    @param clustHsp: name of the clustering program (Grouper, Recon or Piler)
    @type clustHsp: string

    @param multAlign: name of the multiple alignment program MSA (Map, Mafft, Muscle, Tcoffee, ClustalW or Prank)
    @type multAlign: string
    """

    if verbose > 0:
        print "beginning of step 5"
        if smplAlign == 'Blaster':
            print "self-alignment with %s" % smplAlign
        if isStructuralSearch:
            print "structural detection with LTRharvest"
        if isStructuralSearch and clustHsp != "Blastclust":
            print "clustering with %s and Blastclust" % clustHsp
        else:
            print "clustering with %s" % clustHsp
        print "multiple alignment with %s" % multAlign
        sys.stdout.flush()

    if verbose > 0:
        print "detect features on the consensus (step 1 of TEclassifier)"
        sys.stdout.flush()

    if isStructuralSearch and smplAlign == "Blaster":
        prefix = "%s_%s_%s_Struct_%s" % ( projectName, smplAlign, clustHsp, multAlign )
        projectNameDetectFeature = projectName
    else:
        prefix = "%s_%s_%s_%s" % ( projectName, smplAlign, clustHsp, multAlign )
        if isStructuralSearch:
            projectNameDetectFeature = "%s_struct" % projectName
        else:
            projectNameDetectFeature = "%s_sim" % projectName

    isStructDir = os.path.exists("%s_LTRharvest_Blastclust_%s" % ( projectName, multAlign ))
    if isStructuralSearch and not isStructDir:
        print "ERROR: run step 4 struct before step 5"
        sys.exit(1)
        
    clustHspList = []    
    if "Grouper" in clustHsp or "Grp" in clustHsp:
        clustHspList.append("Grouper")
    if "Recon" in clustHsp or "Rec" in clustHsp:
        clustHspList.append("Recon")
    if "Piler" in clustHsp or "Pil" in clustHsp:
        clustHspList.append("Piler")    
    
    for clust in clustHspList:    
        if not os.path.exists("%s_%s_%s_%s" % ( projectName, smplAlign, clust, multAlign )):
            print "ERROR: run step 4 %s before step 5" % clust
            sys.exit(1)

    if os.path.exists("%s_TEclassif" % prefix):
        print "ERROR: directory '%s_TEclassif/' already exists" % prefix
        sys.exit(1)
    os.mkdir("%s_TEclassif" % prefix)
    os.chdir("%s_TEclassif" % prefix)

    if not os.path.exists( "detectFeatures" ):
        os.mkdir( "detectFeatures" )
    else:
        if verbose > 0:
            print "** Warning: directory detectFeatures/ already exists"
            sys.stdout.flush()
    os.chdir( "detectFeatures" )

    if not os.path.exists( configFileName ):
        if not os.path.exists( "../../" + configFileName ):
            print "ERROR: %s not in project directory" % ( configFileName )
            sys.exit(1)
        else:
            os.system( "ln -s ../../%s ." % ( configFileName ) )
            
    sectionName = "detect_features"
    
    try:
        CheckerUtils.checkSectionInConfigFile(config, sectionName) 
    except NoSectionError:
        print "ERROR: the section " + sectionName + " must be define in your config file : " + configFileName
        sys.exit(1)
    
    isOptionErrorRaised = False
    
    optionName = "TE_BLRtx"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True
        
    optionName = "TE_BLRn"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True
        
    optionName = "TE_BLRx"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True
        
    optionName = "HG_BLRn"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True
        
    option = "rDNA_BLRn"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile( config, sectionName, option )
    except NoOptionError:
        print "ERROR: the option '" + option + "' must be define in section '" + sectionName +"' in your config file '" + configFileName + "'"
        isOptionErrorRaised = True
        
    optionName = "TE_hmmer"
    try:
        CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
    except NoOptionError:
        print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
        isOptionErrorRaised = True
        
    if isOptionErrorRaised:
        sys.exit(1)
        
    if (config.get(sectionName, "TE_BLRtx") == "yes" or config.get(sectionName, "TE_BLRn") == "yes"):
        optionName = "TE_nucl_bank"
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
        except NoOptionError:
            print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
        TE_nucl_bank = config.get(sectionName, "TE_nucl_bank")
        if not os.path.exists( TE_nucl_bank ):
            if not os.path.exists( "../../" + TE_nucl_bank ):
                print "ERROR: %s not in project directory" % ( TE_nucl_bank )
                sys.exit(1)
            else:
                os.system( "ln -s ../../%s ." % ( TE_nucl_bank ) )

    if config.get(sectionName, "TE_BLRx") == "yes":
        optionName = "TE_prot_bank"
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
        except NoOptionError:
            print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
        TE_prot_bank = config.get(sectionName, "TE_prot_bank")
        if not os.path.exists( TE_prot_bank ):
            if not os.path.exists( "../../" + TE_prot_bank ):
                print "ERROR: %s not in project directory" % ( TE_prot_bank )
                sys.exit(1)
            else:
                os.system( "ln -s ../../%s ." % ( TE_prot_bank ) )

    if config.get(sectionName, "HG_BLRn") == "yes":
        optionName = "HG_nucl_bank"
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
        except NoOptionError:
            print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
        HG_nucl_bank = config.get(sectionName, "HG_nucl_bank")
        if not os.path.exists( HG_nucl_bank ):
            if not os.path.exists( "../../" + HG_nucl_bank ):
                print "ERROR: %s not in project directory" % ( HG_nucl_bank )
                sys.exit(1)
            else:
                os.system( "ln -s ../../%s ." % ( HG_nucl_bank ) )

    if config.get(sectionName, "te_hmmer") == "yes":
        optionName = "te_hmm_profiles"
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
        except NoOptionError:
            print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
        HMM_profiles = config.get(sectionName, "te_hmm_profiles")
        if not os.path.exists( HMM_profiles ):
            if not os.path.exists( "../../" + HMM_profiles ):
                print "ERROR: %s not in project directory" % ( HMM_profiles )
                sys.exit(1)
            else:
                os.system( "ln -s ../../%s ." % ( HMM_profiles ) )
                
    if config.get(sectionName, "rDNA_BLRn") == "yes":
        optionName = "rDNA_bank"
        try:
            CheckerUtils.checkOptionInSectionInConfigFile(config, sectionName, optionName) 
        except NoOptionError:
            print "ERROR: the option " + optionName + " must be define in " + sectionName +" in your config file : " + configFileName
            sys.exit(1)
        rDNA_bank = config.get(sectionName, "rDNA_bank")
        if not os.path.exists( rDNA_bank ):
            if not os.path.exists( "../../" + rDNA_bank ):
                print "ERROR: %s not in project directory" % ( rDNA_bank )
                sys.exit(1)
            else:
                os.system("ln -s ../../%s ." % rDNA_bank)

    if not os.path.exists("%s_consensus.fa" % prefix):
        # shorten the headers of the consensus
        cDir = os.getcwd()
        if config.get(sectionName, "tmpDir" ) != "":
            tmpDir = config.get(sectionName, "tmpDir")
        else:
            tmpDir = cDir
            
        queue = config.get(sectionName, "resources")
        groupid = "%s_TEdenovo_S5_shortenHeader" % projectName
        acronym = "shortenHeaderConsensus"
        jobdb = RepetJob()
        iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
        lCmdFinish = []
        lCmds = []
        lCmdsTuples = []
        if isStructuralSearch:
            lCmds.append(getShortenHeaderConsensusCmd(projectName, "LTRharvest", "Blastclust", multAlign, isStructuralSearch, iLauncher))
            outFile = "%s_LTRharvest_Blastclust_%s_consensus_shortH.fa" % (projectName, multAlign)
            lCmdFinish.append("shutil.move(\"%s\", \"%s\")" % (outFile, cDir))
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdFinish = lCmdFinish))
        for clust in clustHspList:
            lCmdFinish = []
            lCmds.append(getShortenHeaderConsensusCmd(projectName, smplAlign, clust, multAlign, False, iLauncher))
            lCmdFinish.append("shutil.move(\"%s_%s_%s_%s_consensus_shortH.fa\", \"%s\")" % (projectName, smplAlign, clust, multAlign, cDir))
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdFinish = lCmdFinish))

        cleanMustBeDone = False
        if config.get(sectionName, "clean") == "yes":
            cleanMustBeDone = True
        acronym += clustHsp
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)

        # concatenate all the consensus
        lFiles = glob.glob( "%s_*_%s_consensus_shortH.fa" % ( projectName, multAlign ) )
        lFiles.sort()
        for resFile in lFiles:
            prg = "cat"
            cmd = prg
            cmd += " %s >> %s_consensus.fa" % (resFile, prefix)
            pL.launch(prg, cmd)

        # clean
        if os.path.exists( "%s_%s_Grouper_%s_consensus_shortH.fa" % ( projectName, smplAlign, multAlign ) ):
            os.system( "rm -f %s_%s_Grouper_%s_consensus_shortH.fa" % ( projectName, smplAlign, multAlign ) )
        if os.path.exists( "%s_%s_Recon_%s_consensus_shortH.fa" % ( projectName, smplAlign, multAlign ) ):
            os.system( "rm -f %s_%s_Recon_%s_consensus_shortH.fa" % ( projectName, smplAlign, multAlign ) )
        if os.path.exists( "%s_%s_Piler_%s_consensus_shortH.fa" % ( projectName, smplAlign, multAlign ) ):
            os.system( "rm -f %s_%s_Piler_%s_consensus_shortH.fa" % ( projectName, smplAlign, multAlign ) )
        if os.path.exists( "%s_LTRharvest_Blastclust_%s_consensus_shortH.fa" % ( projectName, multAlign ) ):
            os.system( "rm -f %s_LTRharvest_Blastclust_%s_consensus_shortH.fa" % ( projectName, multAlign ) )

    # launch TEclassifier.py step 1
    prg = os.environ["REPET_PATH"] + "/bin/TEclassifier.py"
    cmd = prg
    cmd += " -p %s" % projectNameDetectFeature
    cmd += " -c %s" % configFileName
    cmd += " -i %s_consensus.fa" % prefix
    cmd += " -s 1"
    if CheckerUtils.isExecutableInUserPath("hmmscan") and CheckerUtils.isExecutableInUserPath("hmmpress"):
        cmd += " -n"
    cmd += " -v %i" % (verbose - 1)
    pL.launch(prg, cmd)

    os.chdir ("../..")

    if verbose > 0:
        print "step 5 finished successfully"; sys.stdout.flush()

#------------------------------------------------------------------------------

def getShortenHeaderClassifAfterConsensusClassification(projectName, smplAlign, clustHSP, multAlign, isStructuralSearch, iLauncher):
    if isStructuralSearch:
        inFile = "%s/%s_%s_%s_Struct_%s_TEclassif/classifConsensus/%s_%s_%s_Struct_%s_uniq_TEclassifier.fa" % (projectDir, projectName, smplAlign, clustHSP, multAlign, projectName, smplAlign, clustHSP, multAlign)
    else:
        inFile = "%s/%s_%s_%s_%s_TEclassif/classifConsensus/%s_%s_%s_%s_uniq_TEclassifier.fa" % (projectDir, projectName, smplAlign, clustHSP, multAlign, projectName, smplAlign, clustHSP, multAlign)
    if not os.path.exists( inFile ):
        msg = "ERROR: consensus file '%s' doesn't exist" % (inFile.replace("%s/" % projectDir, ""))
        sys.stderr.write( "%s\n" % msg )
        sys.exit(1)
    prg = os.environ["REPET_PATH"] + "/bin/shortenHeaderClassif.py"
    lArgs = []
    lArgs.append("-i %s" % inFile)
    lArgs.append("-c")
    lArgs.append("-p %s" % projectName)
    lArgs.append("-s '_'")
    lArgs.append("-o %s_denovoLibTEs.fa" % projectName)
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand(prg, lArgs)
    
#------------------------------------------------------------------------------

class ClassifConsensus(object):
    
    def __init__(self, smplAlign, clustHsp, multAlign, isStructuralSearch, configFileName, verbose):
        self._smplAlign = smplAlign
        self._clustHsp = clustHsp
        self._multAlign = multAlign
        self._isStructuralSearch = isStructuralSearch
        self._configFileName = configFileName
        self._iConfigValue = None
        self._sectionName = "classif_consensus"
        self._projectName = ""
        self._projectDir = ""
        self._verbose = verbose
        self._clean = False
    
    def run(self):
        if verbose > 0:
            print "beginning of step 6"
            if self._smplAlign == 'Blaster':
                print "self-alignment with %s" % self._smplAlign
            if self._isStructuralSearch:
                print "structural detection with LTRharvest"
            if self._isStructuralSearch and self._clustHsp != "Blastclust":
                print "clustering with %s and Blastclust" % self._clustHsp
            else:
                print "clustering with %s" % self._clustHsp
            print "multiple alignment with %s" % self._multAlign
            print "classify the consensus according to their features (PASTEC)"
            sys.stdout.flush()
        
        self._checkConfig()
        self._projectName = self._iConfigValue.get("project", "project_name")
        self._projectDir = self._iConfigValue.get("project", "project_dir")
        if self._iConfigValue.get(self._sectionName, "clean") == "yes":
            self._clean = True
            
        if self._isStructuralSearch and self._smplAlign == "Blaster":
            self._prefix = "%s_%s_%s_Struct_%s" % (self._projectName, self._smplAlign, self._clustHsp, self._multAlign)
            projectNameDetectFeature = projectName
        else:
            self._prefix = "%s_%s_%s_%s" % (self._projectName, self._smplAlign, self._clustHsp, self._multAlign)
            if self._isStructuralSearch:
                projectNameDetectFeature = "%s_struct" % projectName
            else:
                projectNameDetectFeature = "%s_sim" % projectName

        if not os.path.exists("%s_TEclassif/detectFeatures" % self._prefix):
            print "ERROR: run step 5 before step 6"
            help()
            sys.exit(1)
        os.chdir("%s_TEclassif" % self._prefix)
    
        if os.path.exists("classifConsensus"):
            print "ERROR: directory %s_TEclassif/classifConsensus already exists" % self._prefix
            sys.exit(1)
        os.mkdir("classifConsensus")
        os.chdir("classifConsensus")
        os.symlink("../../%s" % self._configFileName, self._configFileName)
        consensusFileName = "%s_consensus.fa" % self._prefix
        os.symlink("../detectFeatures/%s" % consensusFileName, consensusFileName)
        
        if self._checkFeatureTables(projectNameDetectFeature) == 0:
            message = "ERROR: no table recording TE features exists"
            if self._verbose > 0:
                print message
                sys.stdout.flush()
            sys.exit(1)
            
        #insertion banks in DB
        prg = "%s/bin/LaunchPASTEC.py" % os.environ["REPET_PATH"]
        cmd = prg
        cmd += " -C %s" % configFileName
        cmd += " -P %s" % projectNameDetectFeature
        cmd += " -S 1"
        cmd += " -v %i" % self._verbose
        os.system(cmd)
        
        #split consensus file
        seqPerJob = 0
        minSeqPerJob = 100
        maxNbJobs = float(self._iConfigValue.get("classif_consensus", "limit_job_nb"))
        if maxNbJobs != 0:
            nbSeq = FastaUtils.dbSize(consensusFileName)
            message = "nb of sequences: %i" % nbSeq
            if self._verbose > 0:
                print message
                sys.stdout.flush()
            seqPerJob = nbSeq / maxNbJobs
        prg = "%s/bin/dbSplit.py" % os.environ["REPET_PATH"]
        cmd = prg
        cmd += " -i %s" % consensusFileName
        if seqPerJob <= 1.0:
            cmd += " -n %i" % minSeqPerJob
        else:
            cmd += " -n %i" % (seqPerJob + 1)
        cmd += " -d"
        os.system(cmd)
        
        #launch PASTEC on each batch
        message = "launch PASTEC on each batch"
        if self._verbose > 0:
            print message
            sys.stdout.flush()
            
        queue = self._iConfigValue.get(self._sectionName, "resources")
        cDir = os.getcwd()
        if self._iConfigValue.get(self._sectionName, "tmpDir") != "":
            tmpDir = self._iConfigValue.get(self._sectionName, "tmpDir")
        else:
            tmpDir = cDir
            
        groupid = "%s_TEdenovo_ClassifConsensus_PASTEC" % projectNameDetectFeature
        acronym = "PASTEC"
        iDb = DbFactory.createInstance()
        iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(iTJA, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
        lCmdsTuples = []
        lFiles = FileUtils.getFileNamesList("%s/batches" % cDir, "batch_")
        if len(lFiles) == 0:
            print "ERROR: directory 'batches' is empty"
            sys.exit(1)
        classifFile = "%s.classif" % projectNameDetectFeature
        count = 0
        for file in lFiles:
            count += 1
            lCmds = [self.getPASTECcommand(projectNameDetectFeature, iLauncher, file)]    
            lCmdStart = []
            lCmdStart.append("shutil.copy(\"%s/batches/%s\", \".\")" % (cDir, file))
            lCmdStart.append("shutil.copy(\"%s/%s\", \".\")" % (cDir, configFileName))
            lCmdFinish = []
            lCmdFinish.append("shutil.move(\"%s\", \"%s/%s_%i\")" % (classifFile, cDir, classifFile, count))
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._clean)  
        #TODO: connection closed in runLauncherForMultipleJobs(). To change ?
#        iDb.close()
        
        FileUtils.catFilesByPattern("%s_*" % classifFile, classifFile)
        if self._clean:
            FileUtils.removeFilesByPattern("%s_*" % classifFile)

        # Gives stats on classification before redundancy removal
        groupid = "%s_TEdenovo_ClassifConsensus_statsPASTEC" % projectNameDetectFeature
        acronym = "StatPastec"
        iDb = DbFactory.createInstance()
        iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(iTJA, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
        lCmdsTuples = []
        lCmds = [self.getStatPASTECcommand(iLauncher, classifFile)]    
        lCmdStart = []
        lCmdStart.append("shutil.copy(\"%s/%s\", \".\")" % (cDir, classifFile))
        lCmdFinish = []
        lCmdFinish.append("shutil.move(\"%s_stats.txt\", \"%s/.\")" % (classifFile, cDir))
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._clean)  
        
        # Remove redundancy
        iRemoveRedundancy = RemoveRedundancyBasedOnClassif(consensusFileName, classifFile, self._configFileName, self._verbose)
        iRemoveRedundancy.run()
        withoutRddyFastaFileName = "%s_withoutRedundancy.fa" % projectNameDetectFeature
        shutil.move("%s_withoutRedundancy.fa" % self._projectName, withoutRddyFastaFileName)
        
        #TODO: launch GetClassifUniq.py, StatPastec.py, RenameHeaderClassif.py and ReverseComplementAccordingToClassif.py in one job ?
        # Update classif file after redundancy removal
        iGetClassifuniq = GetClassifUniq(withoutRddyFastaFileName, classifFile, self._verbose)
        iGetClassifuniq.run()
        withoutRddyClassifFileName = "%s_withoutRedundancy.classif" % projectNameDetectFeature
        
        # Gives stats on classification after redundancy removal
        groupid = "%s_TEdenovo_ClassifConsensus_statsPASTEC_withoutRddcy" % projectNameDetectFeature
        acronym = "StatPastec"
        iDb = DbFactory.createInstance()
        iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(iTJA, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
        lCmdsTuples = []
        lCmds = [self.getStatPASTECcommand(iLauncher, withoutRddyClassifFileName)]    
        lCmdStart = []
        lCmdStart.append("shutil.copy(\"%s/%s\", \".\")" % (cDir, withoutRddyClassifFileName))
        lCmdFinish = []
        lCmdFinish.append("shutil.move(\"%s_stats.txt\", \"%s/.\")" % (withoutRddyClassifFileName, cDir))
        lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._clean) 

        # Add Wicker code and short headers
        iRHC = RenameHeaderClassif(withoutRddyClassifFileName, withoutRddyFastaFileName, self._projectName, True, True)
        iRHC.setOutputFileName("")
        iRHC.run()
        
        # ReverseComplement
        iRevComplAccording2Classif = ReverseComplementAccordingToClassif()
        iRevComplAccording2Classif.setFastaFile("%s_WickerH.fa" % os.path.splitext(withoutRddyFastaFileName)[0])
        iRevComplAccording2Classif.setClassifFile("%s_WickerH.classif" % os.path.splitext(withoutRddyClassifFileName)[0])
        iRevComplAccording2Classif.run()
        
        shutil.move("%s_WickerH_negStrandReversed.fa" % os.path.splitext(withoutRddyFastaFileName)[0], "%s_denovoLibTEs.fa" % self._projectName)

        if self._verbose > 0:
            print "step 6 finished successfully"
            sys.stdout.flush()
        
    def _checkConfig(self):
        iConfigRules = ConfigRules()
        iConfigRules.addRuleSection(self._sectionName, True)
        iConfigRules.addRuleOption(self._sectionName, "min_redundancy_identity", True)
        iConfigRules.addRuleOption(self._sectionName, "min_redundancy_coverage", True)
        iConfigRules.addRuleOption(self._sectionName, "max_profiles_evalue", True)
        iConfigRules.addRuleOption(self._sectionName, "max_blastn_evalue", True)
        iConfigRules.addRuleOption(self._sectionName, "min_blast_coverage", True)
        iConfigRules.addRuleOption(self._sectionName, "min_profiles_coverage", True)
        iConfigRules.addRuleOption(self._sectionName, "min_SSR_coverage", True)
        iConfigRules.addRuleOption(self._sectionName, "clean", True)
        iConfigRules.addRuleOption(self._sectionName, "tmpDir", True)
        iConfigRules.addRuleOption(self._sectionName, "resources", True)
        
        iConfigChecker = ConfigChecker(self._configFileName, iConfigRules)
        self._iConfigValue = iConfigChecker.getConfig()

    ## Check if the different tables recording sequence features (tandem repeats, matches with known TEs...) exist or not.
    def _checkFeatureTables(self, projectNameDetectFeature):
        exist = 0
        tableSuffix = ["_TR_set",
                       "_SSRtrf_set",
                       "_polyA_set",
                       "_ORF_map",
                       "_TE_BLRn_path",
                       "_TE_BLRtx_path",
                       "_TE_BLRx_path", 
                       "_RepbaseNT_seq",
                       "_RepbaseAA_seq",
                       "_HG_BLRn_path",
                       "_rDNA_BLRn_path",
                       "_Profiles_path",
                       "_ProfilesDB_map"]
        iDb = DbFactory.createInstance()
        for suffix in tableSuffix:
            if iDb.doesTableExist(projectNameDetectFeature + suffix):
                message = "table '%s%s' exists" % (projectNameDetectFeature, suffix)
                message += " (%i lines)" % (iDb.getSize(projectNameDetectFeature + suffix))
                if self._verbose > 0:
                    print message
                    sys.stdout.flush()
                exist += 1
            else:
                message = "table '%s%s' doesn't exist" % (projectNameDetectFeature, suffix)
                if self._verbose > 0:
                    print message
                    sys.stdout.flush()
        iDb.close()
        return exist
    
    def getPASTECcommand(self, projectNameDetectFeature, iLauncher, fileName):
        lArgs = []
        lArgs.append("-C %s" % self._configFileName)
        lArgs.append("-P %s" % projectNameDetectFeature)
        lArgs.append("-S 2")
        lArgs.append("-i %s" % fileName)
        lArgs.append("-v %s" % self._verbose)
        return iLauncher.getSystemCommand("%s/bin/LaunchPASTEC.py" % os.environ["REPET_PATH"], lArgs)
    
    def getStatPASTECcommand(self, iLauncher, classifFileName):
        lArgs = []
        lArgs.append("-i %s" % classifFileName)
        lArgs.append("-o %s_stats.txt" % classifFileName)
        lArgs.append("-v %i" % (self._verbose - 1))
        return iLauncher.getSystemCommand("%s/PASTEC/StatPastec.py" % os.environ["REPET_PATH"], lArgs)
    
#------------------------------------------------------------------------------

def getFilterClassifiedSequencesCmd(inFile, config, sectionName, iLauncher):
    lArgs = []
    lArgs.append("-i %s" % inFile)
    if config.get(sectionName, "filter_SSR") == "yes":
        lArgs.append("-S")
        lArgs.append("-s %s" % config.get(sectionName, "length_SSR"))
    if config.get(sectionName, "filter_noCat_built_from_less_than_10_fragments") == "yes":
        lArgs.append("-N -n 10")
    if config.get(sectionName, "filter_host_gene") == "yes":
        lArgs.append("-g")
    if config.get(sectionName, "filter_potential_chimeric") == "yes":  
        lArgs.append("-c")
    if config.get(sectionName, "filter_incomplete") == "yes":
        lArgs.append("-u")
    lArgs.append("-v %i" % (verbose - 1))
 
    return iLauncher.getSystemCommand("%s/bin/FilterClassifiedSequences.py" % os.environ["REPET_PATH"], lArgs)

#------------------------------------------------------------------------------

def filterConsensus( smplAlign, clustHsp, multAlign, isStructuralSearch ):

    """
    step 7: filter the consensus no SSR and no noCat

    @param smplAlign: name of the alignment program (Blaster or Pals)
    @type smplAlign: string

    @param clustHsp: name of the clustering program (Grouper, Recon or Piler)
    @type clustHsp: string

    @param multAlign: name of the multiple alignment program MSA (Map, Mafft, Muscle, Tcoffee, ClustalW or Prank)
    @type multAlign: string
    """

    if verbose > 0:
        print "beginning of step 7"
        if smplAlign == 'Blaster':
            print "self-alignment with %s" % smplAlign
        if isStructuralSearch:
            print "structural detection with LTRharvest"
        if isStructuralSearch and clustHsp != "Blastclust":
            print "clustering with %s and Blastclust" % clustHsp
        else:
            print "clustering with %s" % clustHsp
        print "multiple alignment with %s" % multAlign
        print "filter the consensus (SSR and noCat)"
        sys.stdout.flush()

    if isStructuralSearch and smplAlign == "Blaster":
        prefix = "%s_%s_%s_Struct_%s" % ( projectName, smplAlign, clustHsp, multAlign )
    else:
        prefix = "%s_%s_%s_%s" % ( projectName, smplAlign, clustHsp, multAlign )

    if not os.path.exists("%s_TEclassif/classifConsensus" % prefix):
        print "ERROR: run step 6 before step 7"
        help()
        sys.exit(1)
    else:
        os.mkdir("%s_TEclassif_Filtered" % prefix)
        os.chdir("%s_TEclassif_Filtered" % prefix)

    if not os.path.exists( "%s/%s" % (projectDir, configFileName) ):
        print "ERROR: %s not in project directory" % configFileName
        sys.exit(1)
    else:
        os.symlink("%s/%s" % (projectDir, configFileName), configFileName)

    sectionName = "filter_consensus"
    fileName = "%s_denovoLibTEs.fa" % projectName
    pathAndFileName = "%s/%s_TEclassif/classifConsensus/%s" % (projectDir, prefix, fileName)
    if not os.path.exists(pathAndFileName):
        msg = "ERROR: fasta file '%s' doesn't exist" % pathAndFileName
        sys.stderr.write("%s\n" % msg)
        sys.exit(1)

    queue = config.get(sectionName, "resources")
    cDir = os.getcwd()
    if config.get(sectionName, "tmpDir" ) != "":
        tmpDir = config.get(sectionName, "tmpDir")
    else:
        tmpDir = cDir
        
    groupid =  "%s_TEdenovo_S7_filterConsensus" % projectName
    acronym = "filterConsensus"
    jobdb = RepetJob()
    iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
    lCmds = []
    lCmds.append(getFilterClassifiedSequencesCmd(fileName, config, sectionName, iLauncher))
    lCmdStart = []
    lCmdStart.append("shutil.copy(\"%s\", \".\")" % pathAndFileName)
    lCmdFinish = []
    lCmdFinish.append("shutil.move(\"%s.filtered\", \"%s/%s_denovoLibTEs_filtered.fa\")" % (fileName, cDir, projectName))
    lCmdsTuples = []
    lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
    cleanMustBeDone = False
    if config.get(sectionName, "clean") == "yes":
        cleanMustBeDone = True
    iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)
    
    os.chdir ("../..")

    if verbose > 0:
        print "step 7 finished successfully"
        sys.stdout.flush()
        
#------------------------------------------------------------------------------

def getLaunchBlastclustCmd(filteredFileName, sectionName, iLauncher):
    lArgs = []
    lArgs.append("-i %s" % filteredFileName)
    lArgs.append("-S %s" % config.get(sectionName, "identity"))
    lArgs.append("-L %s" % config.get(sectionName, "coverage"))
    lArgs.append("--TEdenovoHeader")
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand("%s/bin/LaunchBlastclust.py" % os.environ["REPET_PATH"], lArgs)

def getGiveInfoBlastclustCmd(fileName, iLauncher):
    lArgs = []
    lArgs.append("-i %s" % fileName)
    lArgs.append("-f blastclust")
    lArgs.append("-v %i" % (verbose - 1))
    return iLauncher.getSystemCommand("%s/bin/GiveInfoBlastclust.py" % os.environ["REPET_PATH"], lArgs)

#------------------------------------------------------------------------------

def clusteringConsensus( smplAlign, clustHsp, multAlign, isStructuralSearch ):

    """
    step 8: clustering the consensus 
    
    @param smplAlign: name of the alignment program (Blaster or Pals)
    @type smplAlign: string

    @param clustHsp: name of the clustering program (Grouper, Recon or Piler)
    @type clustHsp: string

    @param multAlign: name of the multiple alignment program MSA (Map, Mafft, Muscle, Tcoffee, ClustalW or Prank)
    @type multAlign: string
    """
    
    if verbose > 0:
        print "beginning of step 8"
        if smplAlign == 'Blaster':
            print "self-alignment with %s" % smplAlign
        if isStructuralSearch:
            print "structural detection with LTRharvest"
        if isStructuralSearch and clustHsp != "Blastclust":
            print "clustering with %s and Blastclust" % clustHsp
        else:
            print "clustering with %s" % clustHsp
        print "multiple alignment with %s" % multAlign
        print "clustering the filtered consensus"
        sys.stdout.flush()

    if isStructuralSearch and smplAlign == "Blaster":
        prefix = "%s_%s_%s_Struct_%s" % ( projectName, smplAlign, clustHsp, multAlign )
    else:
        prefix = "%s_%s_%s_%s" % ( projectName, smplAlign, clustHsp, multAlign )
    
    if not os.path.exists("%s_TEclassif_Filtered/%s_denovoLibTEs_filtered.fa" % (prefix, projectName)):
        print "ERROR: run step 7 before step 8"
        help()
        sys.exit(1)
    else:
        os.mkdir("%s_TEclassif_Filtered_Clustered" % prefix)
        os.chdir("%s_TEclassif_Filtered_Clustered" % prefix)
        
    if not os.path.exists( "%s/%s" % (projectDir, configFileName) ):
        print "ERROR: %s not in project directory" % configFileName
        sys.exit(1)
    else:
        os.symlink("%s/%s" % (projectDir, configFileName), configFileName)

    sectionName = "cluster_consensus"

    filteredFileName = "%s_denovoLibTEs_filtered.fa" % projectName
    absPathFilteredFile = os.path.abspath("../%s_TEclassif_Filtered/%s" % (prefix, filteredFileName))

    queue = config.get(sectionName, "resources")
    cDir = os.getcwd()
    if config.get(sectionName, "tmpDir" ) != "":
        tmpDir = config.get(sectionName, "tmpDir")
    else:
        tmpDir = cDir
        
    groupid = "%s_TEdenovo_S8_blastclust" % projectName
    acronym = "LaunchBlastclust"
    jobdb = RepetJob()
    iLauncher = Launcher(jobdb, os.getcwd(), "", "", cDir, tmpDir, "jobs", queue, groupid)
    lCmds = []
    lCmds.append(getLaunchBlastclustCmd(filteredFileName, sectionName, iLauncher))
    blastclustOutFileName = "%s_blastclust.txt" % filteredFileName
    lCmds.append(getGiveInfoBlastclustCmd(blastclustOutFileName, iLauncher))
    lCmdStart = []
    lCmdStart.append("shutil.copy(\"%s\", \".\")" % absPathFilteredFile)
    lCmdFinish = []
    lCmdFinish.append("shutil.move(\"%s_blastclust.fa\", \"%s/%s_denovoLibTEs_filtered_clustered.fa\")" % (filteredFileName, cDir, projectName))
    lCmdFinish.append("shutil.move(\"GiveInfoBlastclust_%s.log\", \"%s/%s_denovoLibTEs_filtered_blastclustInfo.txt\")" % (blastclustOutFileName, cDir, projectName))
    lCmdsTuples = []
    lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
    cleanMustBeDone = False
    if config.get(sectionName, "clean") == "yes":
        cleanMustBeDone = True
    iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, cleanMustBeDone)    
    
    os.chdir ("../..")

    if verbose > 0:
        print "step 8 finished successfully"
        sys.stdout.flush()
        
if __name__ == "__main__":
    main()