from pyRepetUnit.commons.sql.DbFactory import DbFactory
from pyRepetUnit.commons.sql.TableJobAdaptatorFactory import TableJobAdaptatorFactory
from pyRepetUnit.commons.launcher.Launcher import Launcher
from pyRepetUnit.commons.seq.BioseqDB import BioseqDB
from pyRepetUnit.commons.utils.FileUtils import FileUtils
from ConfigParser import ConfigParser
import sys
import os
import shutil

class RemoveRedundancyBasedOnClassif(object):
    
    def __init__(self, inputFastaName, inputClassifName, configFileName = "", verbose = 0, projectName = "rmvRddy", identityTh = 0.95, coverageTh = 0.98, clean = True, outFileName = ""):
        self._inputFastaName = inputFastaName
        self._inputClassifName = inputClassifName
        self._configFileName = configFileName
        self._projectName = projectName
        self._verbose = int(verbose)
        self._outFileName = outFileName
        self._identityThreshold = identityTh
        self._coverageThreshold = coverageTh
        self._dOrderAndCompletion = {}
        self._lClassifNames = ["LTR", "LARD", "TRIM", "DIRS", "PLE", "LINE", "SINE", "TIR", "MITE", "Crypton", "Helitron", "Maverick"]
        self._clean = clean
        
    def setProjectName(self, projectName):
        self._projectName = projectName
        
    def setIdentityThreshold(self, identityTh):
        self._identityThreshold =identityTh
        
    def setCoverageThreshold(self, coverageTh):
        self._coverageThreshold = coverageTh
        
    def setClean(self, clean):
        self._clean = clean
        
    def setOutFileName(self, fileName):
        self._outFileName = fileName
    
    def setDOrderAndCompletion(self, dict):
        self._dOrderAndCompletion = dict
    
    def setDOrderInInputFile(self, dict):
        self._dOrderInInputFile = dict
    
    def run(self):
        workingDir = "removeRedundancy"
        if os.path.exists(workingDir):
            print "ERROR: %s already exists." % workingDir
            sys.exit(1)
        os.mkdir(workingDir)
        os.chdir(workingDir)
        os.symlink("../%s" % self._inputFastaName, self._inputFastaName)
        os.symlink("../%s" % self._inputClassifName, self._inputClassifName)
        if self._configFileName != "" and FileUtils.isRessourceExists("../%s" % self._configFileName):
            os.symlink("../%s" % self._configFileName, self._configFileName)
            self._setAttributesFromConfig()
        self._splitByOrderAndCompletion()
        self._removeRedundancyFilteringByOrderAndCompletion()
        self._removeRedundancyIncompleteConsensusIncludeInCompleteConsensusByOrder()
        self._removeRedundancySpecificClassif()
        self._removeRedundancyNoCatIncludeInClassifiedConsensus()
        self._concatenateFileWithoutRedundancySplitedByOrderAndCompletion()
        os.chdir("..")
        shutil.copy("%s/%s" % (workingDir, self._outFileName), self._outFileName)
        if self._clean:
            shutil.rmtree(workingDir)
            
    def _concatenateFileWithoutRedundancySplitedByOrderAndCompletion(self):
        if self._outFileName == "":
            self._outFileName = "%s_withoutRedundancy.fa" % self._projectName
        if os.path.exists(self._outFileName):
            os.remove(self._outFileName)           
        FileUtils.catFilesByPattern("%s_*_uniq.fa" % self._projectName, self._outFileName)
                
    def _removeRedundancyFilteringByOrderAndCompletion(self):
        groupid = "%s_rmvRddcy_sameBank" % self._projectName
        acronym = "rmvRedundancy"
        if self._configFileName != "":
            iDb = DbFactory.createInstance(self._configFileName)
        else:
            iDb = DbFactory.createInstance()
        iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(iTJA, os.getcwd(), "", "", self._cDir, self._tmpDir, "jobs", self._resources, groupid, acronym)
        lCmdsTuples = []
        for orderAndCompletion in self._dOrderAndCompletion:
            lCmds = []
            queryFileName = "%s_%s.fa" % (self._projectName, orderAndCompletion)
            subjectFileName = queryFileName
            outFileName = "%s_%s_uniq.fa" % (self._projectName, orderAndCompletion) 
            lCmds.append(self._getRemoveRedundancyCmd(queryFileName, subjectFileName, outFileName, iLauncher))
            lCmdStart = []
            lCmdStart.append("shutil.copy(\"%s/%s\", \".\")" % (self._cDir, queryFileName))
            lCmdFinish = []
            lCmdFinish.append("shutil.move(\"%s\", \"%s/.\" )" % (outFileName, self._cDir))
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        if lCmdsTuples != []:
            iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._clean)
        else:
            iDb.close()
            
    def _removeRedundancyIncompleteConsensusIncludeInCompleteConsensusByOrder(self):
        groupid = "%s_rmvRddcy_incompInComp" % self._projectName
        acronym = "rmvRedundancy"
        if self._configFileName != "":
            iDb = DbFactory.createInstance(self._configFileName)
        else:
            iDb = DbFactory.createInstance()
        iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(iTJA, os.getcwd(), "", "", self._cDir, self._tmpDir, "jobs", self._resources, groupid, acronym)
        lCmdsTuples = []
        for orderName in self._dOrderInInputFile:
            queryFileName = "%s_%s_incomplete_uniq.fa" % (self._projectName, orderName)
            subjectFileName = "%s_%s_complete_uniq.fa" % (self._projectName, orderName)
            if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
                lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
                lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        if lCmdsTuples != []:
            iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._clean)
        else:
            iDb.close()
        
    def _removeRedundancySpecificClassif(self):
        groupid = "%s_rmvRddcy_specificClassif" % self._projectName
        acronym = "rmvRedundancy"
        if self._configFileName != "":
            iDb = DbFactory.createInstance(self._configFileName)
        else:
            iDb = DbFactory.createInstance()
        iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(iTJA, os.getcwd(), "", "", self._cDir, self._tmpDir, "jobs", self._resources, groupid, acronym)
        
        #comp
        lCmdsTuples = []
        #LARD in LTR
        queryFileName = "%s_LARD_complete_uniq.fa" % self._projectName
        subjectFileName = "%s_LTR_complete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        #TRIM in LTR
        queryFileName = "%s_TRIM_complete_uniq.fa" % self._projectName
        subjectFileName = "%s_LTR_complete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        #SINE in LINE
        queryFileName = "%s_SINE_complete_uniq.fa" % self._projectName
        subjectFileName = "%s_LINE_complete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        queryFileName = "%s_SINE_incomplete_uniq.fa" % self._projectName
        subjectFileName = "%s_LINE_complete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        #MITE in TIR
        queryFileName = "%s_MITE_complete_uniq.fa" % self._projectName
        subjectFileName = "%s_TIR_complete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        if lCmdsTuples != []:
            iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._clean)
        else:
            iDb.close()
        
        if self._configFileName != "":
            iDb = DbFactory.createInstance(self._configFileName)
        else:
            iDb = DbFactory.createInstance()
        iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
        iLauncher = Launcher(iTJA, os.getcwd(), "", "", self._cDir, self._tmpDir, "jobs", self._resources, groupid, acronym)
        #incomp
        lCmdsTuples = []
        #LARD in LTR
        queryFileName = "%s_LARD_complete_uniq.fa" % self._projectName
        subjectFileName = "%s_LTR_incomplete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        #TRIM in LTR
        queryFileName = "%s_TRIM_complete_uniq.fa" % self._projectName
        subjectFileName = "%s_LTR_incomplete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        #SINE in LINE
        queryFileName = "%s_SINE_complete_uniq.fa" % self._projectName
        subjectFileName = "%s_LINE_incomplete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        queryFileName = "%s_SINE_incomplete_uniq.fa" % self._projectName
        subjectFileName = "%s_LINE_incomplete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        #MITE in TIR
        queryFileName = "%s_MITE_complete_uniq.fa" % self._projectName
        subjectFileName = "%s_TIR_incomplete_uniq.fa" % self._projectName
        if os.path.exists(queryFileName) and os.path.exists(subjectFileName):
            lCmds, lCmdStart, lCmdFinish = self._fillCmdsList(queryFileName, subjectFileName, iLauncher)
            lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds, lCmdStart, lCmdFinish))
        if lCmdsTuples != []:
            iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._clean)
        else:
            iDb.close()
    
    def _removeRedundancyNoCatIncludeInClassifiedConsensus(self):
        if "noCat_NA" not in self._dOrderAndCompletion:
            return
        del self._dOrderAndCompletion["noCat_NA"]
        queryFileName = "%s_noCat_NA_uniq.fa" % self._projectName
        if os.path.exists(queryFileName):
            outFileName = "%s.tmp" % queryFileName
            groupid = "%s_rmvRddcy_noCatInClassified" % self._projectName
            acronym = "rmvRedundancy"
            if self._configFileName != "":
                iDb = DbFactory.createInstance(self._configFileName)
            else:
                iDb = DbFactory.createInstance()
            iTJA = TableJobAdaptatorFactory.createInstance(iDb, "jobs")
            iLauncher = Launcher(iTJA, os.getcwd(), "", "", self._cDir, self._tmpDir, "jobs", self._resources, groupid, acronym)
            isRemoveRedundancyCmdInList = False
            lCmds = []
            lCmds.append("shutil.copy(\"%s/%s\", \".\")" % (self._cDir, queryFileName))
            for orderAndCompletion in self._dOrderAndCompletion:
                subjectFileName = "%s_%s_uniq.fa" % (self._projectName, orderAndCompletion)
                if os.path.exists(subjectFileName):
                    isRemoveRedundancyCmdInList = True
                    lCmds.append("shutil.copy(\"%s/%s\", \".\")" % (self._cDir, subjectFileName))
                    lCmds.append(self._getRemoveRedundancyCmd(queryFileName, subjectFileName, outFileName, iLauncher))
                    lCmds.append("if os.path.exists(\"%s\"):" % outFileName)
                    lCmds.append("\tos.remove(\"%s\")" % queryFileName)
                    lCmds.append("\tshutil.move(\"%s\", \"%s\")" % (outFileName, queryFileName))
            lCmds.append("os.remove(\"%s/%s\")" % (self._cDir, queryFileName))
            lCmds.append("shutil.move(\"%s\", \"%s/.\" )" % (queryFileName, self._cDir))
            if isRemoveRedundancyCmdInList:
                lCmdsTuples = []
                lCmdsTuples.append(iLauncher.prepareCommands_withoutIndentation(lCmds))
                iLauncher.runLauncherForMultipleJobs(acronym, lCmdsTuples, self._clean)
            else:
                iDb.close()
        
    def _getRemoveRedundancyCmd(self, queryFileName, subjectFileName, outFileName, iLauncher): 
        lArgs = []
        lArgs.append("-q %s" % queryFileName)
        lArgs.append("-s %s" % subjectFileName)
        lArgs.append("-o %s" % outFileName)
        lArgs.append("-i %s" % self._identityThreshold)
        lArgs.append("-l %s" % self._coverageThreshold)
        if self._clean:
            lArgs.append("-c")
        lArgs.append("-v %i" % (self._verbose - 1))
        return iLauncher.getSystemCommand("%s/bin/rmvRedundancy.py" % os.environ["REPET_PATH"], lArgs)
       
    def _splitByOrderAndCompletion(self):
        iClassif = open(self._inputClassifName,"r")
        iBioseqDB = BioseqDB(self._inputFastaName)
        self._dOrderAndCompletion = {}
        self._dOrderInInputFile = {}
        for classifLine in iClassif.readlines():
            lClassifElements = classifLine.split("\t")
            seqName = lClassifElements[0]
            if lClassifElements[3] != "ok":
                orderAndCompletion = "PotentialChimeric_NA"
            else:
                orderAndCompletion = "%s_%s" % (lClassifElements[5],lClassifElements[6])
                self._dOrderInInputFile[lClassifElements[5]] = None
            key = self._dOrderAndCompletion.get(orderAndCompletion, None)
            if not key:
                self._dOrderAndCompletion[orderAndCompletion] = open("%s_%s.fa" % (self._projectName, orderAndCompletion), "w")  
            iBioseq2 = iBioseqDB.fetch(seqName)       
            iBioseq2.write(self._dOrderAndCompletion[orderAndCompletion])
        iClassif.close()
        for orderAndCompletion in self._dOrderAndCompletion.itervalues():
            orderAndCompletion.close()

    def _setAttributesFromConfig(self):
        iConfig = ConfigParser()
        iConfig.readfp(open(self._configFileName))
        self.setProjectName(iConfig.get("project", "project_name"))
        sectionName = "classif_consensus"
        self.setIdentityThreshold(iConfig.get(sectionName, "min_redundancy_identity"))
        self.setCoverageThreshold(iConfig.get(sectionName, "min_redundancy_coverage"))
        self.setOutFileName("%s_withoutRedundancy.fa" % self._projectName)
        if iConfig.get(sectionName, "clean") == "no":
            self.setClean(False)
        self._resources = iConfig.get(sectionName, "resources")
        self._cDir = os.getcwd()
        if iConfig.get(sectionName, "tmpDir" ) != "":
            self._tmpDir = iConfig.get(sectionName, "tmpDir")
        else:
            self._tmpDir = self._cDir
            
    def _fillCmdsList(self, queryFileName, subjectFileName, iLauncher):
        lCmds = []
        lCmdStart = []
        lCmdFinish = []
        outFileName = "%s.tmp" % queryFileName
        lCmds.append(self._getRemoveRedundancyCmd(queryFileName, subjectFileName, outFileName, iLauncher))
        lCmdStart.append("shutil.copy(\"%s/%s\", \".\")" % (self._cDir, queryFileName))
        lCmdStart.append("shutil.copy(\"%s/%s\", \".\")" % (self._cDir, subjectFileName))
        lCmdFinish.append("if os.path.exists(\"%s\"):" % outFileName)
        lCmdFinish.append("\tos.remove(\"%s/%s\")" % (self._cDir, queryFileName))
        lCmdFinish.append("\tshutil.move(\"%s\", \"%s/%s\")" % (outFileName, self._cDir, queryFileName))
        return lCmds, lCmdStart, lCmdFinish